@@ -1,5 +1,5 @@
 /*
- *	$Id: ml_char.c,v 1.16 2003/09/30 10:24:13 arakiken Exp $
+ *	$Id: ml_char.c,v 1.21 2004/03/23 12:55:45 h_minami Exp $
  */
 
 #include  "ml_char.h"
@@ -39,10 +39,12 @@
 	((color) = ((color) == ML_FG_COLOR ? 0x8 : \
 		(color) == ML_BG_COLOR ? 0x9 : (color) & ~ML_BOLD_COLOR_MASK))
 
+#define  IS_VALID_INTERN_COLOR(color) ((ML_BLACK <=color) && (color <=0x9))
+
 #define  EXTERN_COLOR(color) \
 	((color) = ((color) == 0x8 ? ML_FG_COLOR : \
 		(color) == 0x9 ? ML_BG_COLOR : (color)))
-	
+
 #define  FG_COLOR(attr)  (((attr) >> 11) & 0xf)
 
 #define  SET_FG_COLOR(attr,color)  ((attr) = (((attr) & 0xffff87ff) | ((color) << 11)))
@@ -115,222 +117,6 @@ ml_unuse_multi_col_char(void)
 }
 
 
-/*
- * string functions
- */
- 
-int
-ml_str_init(
-	ml_char_t *  str ,
-	u_int  size
-	)
-{
-	int  count ;
-
-	for( count = 0 ; count < size ; count ++)
-	{
-		ml_char_init( str ++) ;
-	}
-
-	return  1 ;
-}
-
-ml_char_t *
-__ml_str_init(
-	ml_char_t *  str ,	/* alloca()-ed memory (see ml_char.h) */
-	u_int  size
-	)
-{
-	if( str == NULL)
-	{
-		/* alloca() failed. */
-		
-		return  NULL ;
-	}
-
-	if( ! ( ml_str_init( str , size)))
-	{
-		return  NULL ;
-	}
-
-	return  str ;
-}
-	
-ml_char_t *
-ml_str_new(
-	u_int  size
-	)
-{
-	ml_char_t *  str ;
-
-	if( ( str = malloc( sizeof( ml_char_t) * size)) == NULL)
-	{
-	#ifdef  DEBUG
-		kik_warn_printf( KIK_DEBUG_TAG " malloc() failed.\n") ;
-	#endif
-	
-		return  NULL ;
-	}
-
-	if( ml_str_init( str , size) == 0)
-	{
-		free( str) ;
-		
-		return  NULL ;
-	}
-
-	return  str ;
-}
-
-int
-ml_str_final(
-	ml_char_t *  str ,
-	u_int  size
-	)
-{
-	int  count ;
-
-	for( count = 0 ; count < size ; count ++)
-	{
-		ml_char_final( &str[count]) ;
-	}
-
-	return  1 ;
-}
-
-int
-ml_str_delete(
-	ml_char_t *  str ,
-	u_int  size
-	)
-{
-	if( ml_str_final( str , size))
-	{
-		free( str) ;
-
-		return  1 ;
-	}
-	else
-	{
-		free( str) ;
-
-		return  0 ;
-	}
-}
-
-/*
- * dst and src may overlap.
- */
-inline int
-ml_str_copy(
-	ml_char_t *  dst ,
-	ml_char_t *  src ,
-	u_int  size
-	)
-{
-	int  count ;
-
-	if( size == 0 || dst == src)
-	{
-		return  0 ;
-	}
-
-	if( dst < src)
-	{
-		for( count = 0 ; count < size ; count ++)
-		{
-			ml_char_copy( dst++ , src++) ;
-		}
-	}
-	else if( dst > src)
-	{
-		dst += size ;
-		src += size ;
-		for( count = 0 ; count < size ; count ++)
-		{
-			ml_char_copy( --dst  , --src) ;
-		}
-	}
-
-	return  1 ;
-}
-
-inline u_int
-ml_str_cols(
-	ml_char_t *  chars ,
-	u_int  len
-	)
-{
-	int  count ;
-	u_int  cols ;
-
-	cols = 0 ;
-
-	for( count = 0 ; count < len ; count ++)
-	{
-		cols += ml_char_cols( &chars[count]) ;
-	}
-
-	return  cols ;
-}
-
-/*
- * XXX
- * Returns inaccurate result in dealing with combined characters.
- * Even if they have the same bytes, false is returned since
- * ml_char_t:multi_ch-s never point the same address.)
- */
-inline int
-ml_str_equal(
-	ml_char_t *  str1 ,
-	ml_char_t *  str2 ,
-	u_int  len
-	)
-{
-	return  memcmp( str1 , str2 , sizeof( ml_char_t) * len) == 0 ;
-}
-
-inline int
-ml_str_bytes_equal(
-	ml_char_t *  str1 ,
-	ml_char_t *  str2 ,
-	u_int  len
-	)
-{
-	int  count ;
-
-	for( count = 0 ; count < len ; count ++)
-	{
-		if( ! ml_char_bytes_equal( str1 ++ , str2 ++))
-		{
-			return  0 ;
-		}
-	}
-
-	return  1 ;
-}
-
-#ifdef  DEBUG
-
-void
-ml_str_dump(
-	ml_char_t *  chars ,
-	u_int  len
-	)
-{
-	int  count ;
-
-	for( count = 0 ; count < len ; count ++)
-	{
-		ml_char_dump( &chars[count]) ;
-	}
-
-	kik_msg_printf( "\n") ;
-}
-
-#endif
-
-
 /*
  * character functions
  */
@@ -340,10 +126,19 @@ ml_char_init(
 	ml_char_t *  ch
 	)
 {
-	memset( ch , 0 , sizeof( ml_char_t)) ;
-	
-	/* u.ch.is_single_ch is true */
-	ch->u.ch.attr = 0x1 ;
+	if( sizeof( ml_char_t *) != sizeof( ml_char_t))
+	{
+		/*ILP32*/
+		memset( ch , 0 , sizeof( ml_char_t)) ;
+		/* set u.ch.is_single_ch */
+		ch->u.ch.attr = 0x1 ;
+	}
+	else
+	{
+		/*LP64*/
+		/* LSB of multi_ch must be "is_single_ch"  */
+		ch->u.multi_ch =(ml_char_t *)0x1 ;
+	}
 
 	return  1 ;
 }
@@ -382,7 +177,42 @@ ml_char_set(
 
 	INTERN_COLOR(fg_color) ;
 	INTERN_COLOR(bg_color) ;
-	
+
+#ifdef  DEBUG
+	if( size > MAX_CHAR_SIZE)
+	{
+		kik_warn_printf(KIK_DEBUG_TAG "size(%d) too large\n", size) ;
+	}
+	if( cs >= MAX_CHARSET)
+	{
+		kik_warn_printf(KIK_DEBUG_TAG "charset(%x) is out of range\n", cs) ;
+	}
+	if( is_biwidth > 1 || is_biwidth < 0)
+	{
+		kik_warn_printf(KIK_DEBUG_TAG "is_biwidth should be 0/1\n") ;
+	}
+	if( is_comb > 1 || is_comb < 0)
+	{
+		kik_warn_printf(KIK_DEBUG_TAG "is_comb should be 0/1\n") ;
+	}
+	if( !IS_VALID_INTERN_COLOR(fg_color))
+	{
+		kik_warn_printf(KIK_DEBUG_TAG "fg_color is not valid\n") ;
+	}
+	if( !IS_VALID_INTERN_COLOR(bg_color))
+	{
+		kik_warn_printf(KIK_DEBUG_TAG "bg_color is not valid\n") ;
+	}
+	if( is_bold > 1 || is_bold < 0)
+	{
+		kik_warn_printf(KIK_DEBUG_TAG "is_bold should be 0/1\n") ;
+	}
+	if( is_underlined > 1 || is_underlined < 0)
+	{
+		kik_warn_printf(KIK_DEBUG_TAG "is_underlined should be 0/1\n") ;
+	}
+#endif
+
 	ch->u.ch.attr =
 		COMPOUND_ATTR(size,cs,is_biwidth,fg_color,bg_color,is_bold,is_underlined,is_comb) ;
 
@@ -430,10 +260,10 @@ ml_char_combine(
 			return  0 ;
 		}
 
-		ml_str_init( multi_ch , 2) ;
-
+		ml_char_init( multi_ch) ;
 		ml_char_copy( multi_ch , ch) ;
 
+		ml_char_init( multi_ch + 1) ;
 		if( ml_char_set( multi_ch + 1 , bytes , size , cs , is_biwidth , is_comb ,
 			fg_color , bg_color , is_bold , is_underlined) == 0)
 		{
@@ -920,7 +750,39 @@ ml_char_restore_color(
 	}
 }
 
-int
+/* XXX only used in iscii_logical() */
+inline int
+ml_char_copy_color_reversed_flag(
+	ml_char_t *  dst ,
+	ml_char_t *  src
+	)
+{
+	if( IS_SINGLE_CH(src->u.ch.attr))
+	{
+		if( IS_REVERSED(src->u.ch.attr))
+		{
+			return  ml_char_reverse_color( dst) ;
+		}
+		else
+		{
+			return  ml_char_restore_color( dst) ;
+		}
+	}
+	else
+	{
+		/* See the first character only */
+		if( IS_REVERSED(src->u.multi_ch->u.ch.attr))
+		{
+			return  ml_char_reverse_color( dst) ;
+		}
+		else
+		{
+			return  ml_char_restore_color( dst) ;
+		}
+	}
+}
+
+inline int
 ml_char_is_null(
 	ml_char_t *  ch
 	)
@@ -1023,7 +885,7 @@ ml_char_bytes_equal(
 	return  1 ;
 }
 
-ml_char_t *
+inline ml_char_t *
 ml_sp_ch(void)
 {
 	if( sp_ch.u.ch.attr == 0)
@@ -1035,7 +897,7 @@ ml_sp_ch(void)
 	return  &sp_ch ;
 }
 
-ml_char_t *
+inline ml_char_t *
 ml_nl_ch(void)
 {
 	if( nl_ch.u.ch.attr == 0)
