@@ -1,12 +1,12 @@
 /*
- *	$Id: ml_line.h,v 1.10 2003/08/09 15:31:49 arakiken Exp $
+ *	$Id: ml_line.h,v 1.14 2004/03/04 15:59:02 arakiken Exp $
  */
 
 #ifndef  __ML_LINE_H__
 #define  __ML_LINE_H__
 
 
-#include  "ml_char.h"
+#include  "ml_str.h"
 #include  "ml_shape.h"
 #include  "ml_bidi.h"		/* ml_bidi_state_t */
 
@@ -20,7 +20,7 @@ enum
 
 
 /*
- * this object size should be kept as small as possible.
+ * This object size should be kept as small as possible.
  * (160bit ILP32) (224bit ILP64)
  */
 typedef struct  ml_line
