@@ -1,5 +1,5 @@
 /*
- *	$Id: ml_logical_visual.c,v 1.16 2003/09/21 05:27:34 arakiken Exp $
+ *	$Id: ml_logical_visual.c,v 1.19 2004/03/03 12:45:06 arakiken Exp $
  */
 
 #include  "ml_logical_visual.h"
@@ -11,6 +11,14 @@
 
 #define  CURSOR_LINE(logvis)  (ml_model_get_line((logvis)->model,(logvis)->cursor->row))
 
+#if  0
+#define  __DEBUG
+#endif
+
+#if  0
+#define  CURSOR_DEBUG
+#endif
+
 
 typedef struct  container_logical_visual
 {
@@ -900,6 +908,25 @@ search_same_line(
 	return  -1 ;
 }
 
+static int
+copy_color_reversed_flag(
+	ml_line_t *  dst ,
+	ml_line_t *  src
+	)
+{
+	int  col ;
+	u_int  copy_len ;
+	
+	copy_len = K_MIN(src->num_of_filled_chars,dst->num_of_filled_chars) ;
+
+	for( col = 0 ; col < copy_len ; col ++)
+	{
+		ml_char_copy_color_reversed_flag( dst->chars + col , src->chars + col) ;
+	}
+
+	return  1 ;
+}
+
 static int
 iscii_visual(
 	ml_logical_visual_t *  logvis
@@ -908,6 +935,11 @@ iscii_visual(
 	iscii_logical_visual_t *  iscii_logvis ;
 	ml_line_t *  line ;
 	int  row ;
+#ifdef  __DEBUG
+	int  __hit_num = 0 ;
+	int  __mod_num = 0 ;
+	int  __miss_num = 0 ;
+#endif
 
 	if( logvis->is_visual)
 	{
@@ -973,7 +1005,6 @@ iscii_visual(
 	
 	for( row = 0 ; row < logvis->model->num_of_rows ; row ++)
 	{
-		int  is_cache_active ;
 		int  hit_row ;
 
 		line = ml_model_get_line( logvis->model , row) ;
@@ -982,19 +1013,40 @@ iscii_visual(
 		{
 			continue ;
 		}
-		
+
 		if( ( hit_row = search_same_line( iscii_logvis->logical_lines ,
 			iscii_logvis->logical_num_of_rows , row , line)) == -1)
 		{
-			is_cache_active = 0 ;
+		#ifdef  __DEBUG
+			__miss_num ++ ;
+		#endif
+		
+			/* caching */
+			ml_line_copy_line( &iscii_logvis->logical_lines[row] , line) ;
+
+			ml_line_iscii_visual( line , iscii_logvis->iscii_lang) ;
+
+			/* XXX Adhoc implementation for shrinking the amount of memory usage XXX */
+			copy_color_reversed_flag( line , &iscii_logvis->logical_lines[row]) ;
+
+			/* caching */
+			ml_line_copy_line( &iscii_logvis->visual_lines[row] , line) ;
 		}
 		else
 		{
+		#ifdef  __DEBUG
+			__hit_num ++ ;
+		#endif
+
 			if( ml_line_is_modified( line))
 			{
 				int  beg ;
 				int  end ;
 
+			#ifdef  __DEBUG
+				__mod_num ++ ;
+			#endif
+			
 				beg = ml_iscii_convert_logical_char_index_to_visual(
 						&iscii_logvis->visual_lines[hit_row] ,
 						ml_line_get_beg_of_modified(line)) ;
@@ -1013,36 +1065,17 @@ iscii_visual(
 				ml_line_updated( &iscii_logvis->visual_lines[hit_row]) ;
 				ml_line_set_modified( &iscii_logvis->visual_lines[hit_row] , beg , end) ;
 			}
-			
+
 			if( hit_row != row)
 			{
 				ml_line_copy_line( &iscii_logvis->logical_lines[row] , line) ;
 				ml_line_copy_line( &iscii_logvis->visual_lines[row] ,
 					&iscii_logvis->visual_lines[hit_row]) ;
 			}
-						
-			is_cache_active = 1 ;
-		}
-		
-		if( is_cache_active)
-		{
+			
 			/* using cached line */
 			ml_line_copy_line( line , &iscii_logvis->visual_lines[row]) ;
 		}
-		else
-		{
-		#ifdef  __DEBUG
-			kik_debug_printf( KIK_DEBUG_TAG " iscii rendering.\n") ;
-		#endif
-
-			/* caching */
-			ml_line_copy_line( &iscii_logvis->logical_lines[row] , line) ;
-
-			ml_line_iscii_visual( line , iscii_logvis->iscii_lang) ;
-
-			/* caching */
-			ml_line_copy_line( &iscii_logvis->visual_lines[row] , line) ;
-		}
 	}
 
 	iscii_logvis->cursor_logical_char_index = logvis->cursor->char_index ;
@@ -1056,6 +1089,8 @@ iscii_visual(
 #ifdef  __DEBUG
 	kik_debug_printf( KIK_DEBUG_TAG " [col %d index %d]\n" ,
 		logvis->cursor->col , logvis->cursor->char_index) ;
+	kik_debug_printf( KIK_DEBUG_TAG " search hit: %d(mod: %d), miss: %d\n" ,
+		__hit_num , __mod_num , __miss_num) ;
 #endif
 
 	logvis->is_visual = 1 ;
@@ -1080,8 +1115,20 @@ iscii_logical(
 	
 	for( row = 0 ; row < logvis->model->num_of_rows ; row ++)
 	{
-		ml_line_copy_line( ml_model_get_line( logvis->model , row) ,
-			&iscii_logvis->logical_lines[row]) ;
+		ml_line_t *  vis_line ;
+
+		vis_line = ml_model_get_line( logvis->model , row) ;
+
+		/*
+		 * XXX Adhoc implementation for shrinking the amount of memory usage XXX
+		 *
+		 * If logical lines are changed by vt sequences,
+		 * reversed flag will be all cleared.
+		 */
+		copy_color_reversed_flag( &iscii_logvis->logical_lines[row] , vis_line) ;
+		copy_color_reversed_flag( &iscii_logvis->visual_lines[row] , vis_line) ;
+
+		ml_line_copy_line( vis_line , &iscii_logvis->logical_lines[row]) ;
 	}
 
 	logvis->cursor->char_index = iscii_logvis->cursor_logical_char_index ;
