@@ -1,5 +1,5 @@
 /*
- *	$Id: ml_screen.c,v 1.28 2003/09/14 04:05:47 arakiken Exp $
+ *	$Id: ml_screen.c,v 1.34 2004/10/09 04:39:47 arakiken Exp $
  */
 
 #include  "ml_screen.h"
@@ -11,8 +11,7 @@
 #include  <kiklib/kik_util.h>	/* K_MIN */
 
 
-#define  ROW_IN_LOGS( model , row) \
-	( ml_get_num_of_logged_lines( &(model)->logs) + row)
+#define  ROW_IN_LOGS( screen , row) ( ml_get_num_of_logged_lines( &(screen)->logs) + row)
 
 #if  1
 #define  EXIT_BS_AT_BOTTOM
@@ -190,6 +189,33 @@ reverse_or_restore_color(
 	u_int  size_except_spaces ;
 	int  beg_regarding_rtl ;
 
+	row = end_row ;
+	
+	while( ( line = ml_screen_get_line( screen , row)) == NULL || ml_line_is_empty( line))
+	{
+		if( -- row < 0 && abs(row) > ml_get_num_of_logged_lines( &screen->logs))
+		{
+			return  0 ;
+		}
+	}
+
+	if( row < end_row)
+	{
+		if( ml_line_is_rtl( line))
+		{
+			end_char_index = ml_line_beg_char_index_regarding_rtl( line) ;
+		}
+		else
+		{
+			if( ( end_char_index = ml_get_num_of_filled_chars_except_spaces( line)) > 0)
+			{
+				end_char_index -- ;
+			}
+		}
+		
+		end_row = row ;
+	}
+
 	row = beg_row ;
 
 	while( 1)
@@ -349,6 +375,33 @@ check_or_copy_region(
 	int  beg_regarding_rtl ;
 	int  row ;
 
+	row = end_row ;
+	
+	while( ( line = ml_screen_get_line( screen , row)) == NULL || ml_line_is_empty( line))
+	{
+		if( -- row < 0 && abs(row) > ml_get_num_of_logged_lines( &screen->logs))
+		{
+			return  0 ;
+		}
+	}
+
+	if( row < end_row)
+	{
+		if( ml_line_is_rtl( line))
+		{
+			end_char_index = ml_line_beg_char_index_regarding_rtl( line) ;
+		}
+		else
+		{
+			if( ( end_char_index = ml_get_num_of_filled_chars_except_spaces( line)) > 0)
+			{
+				end_char_index -- ;
+			}
+		}
+		
+		end_row = row ;
+	}
+
 	row = beg_row ;
 
 	while( 1)
@@ -1017,7 +1070,7 @@ ml_screen_visual(
 	}
 	else
 	{
-		return  0 ;
+		return  1 ;
 	}
 }
 
@@ -1032,7 +1085,7 @@ ml_screen_logical(
 	}
 	else
 	{
-		return  0 ;
+		return  1 ;
 	}
 }
 
@@ -1999,7 +2052,7 @@ ml_screen_save_cursor(
 	ml_screen_t *  screen
 	)
 {
-	return  ml_cursor_save( screen->edit) ;
+	return  ml_edit_save_cursor( screen->edit) ;
 }
 
 int
@@ -2007,7 +2060,7 @@ ml_screen_restore_cursor(
 	ml_screen_t *  screen
 	)
 {
-	return  ml_cursor_restore( screen->edit) ;
+	return  ml_edit_restore_cursor( screen->edit) ;
 }
 
 int
@@ -2040,6 +2093,14 @@ ml_screen_cursor_invisible(
 	return  1 ;
 }
 
+int
+ml_screen_is_cursor_visible(
+	ml_screen_t *  screen
+	)
+{
+	return screen->is_cursor_visible ;
+}
+
 int
 ml_screen_use_normal_edit(
 	ml_screen_t *  screen
