@@ -1,5 +1,5 @@
 /*
- *	$Id: ml_term.c,v 1.26 2003/09/14 04:05:47 arakiken Exp $
+ *	$Id: ml_term.c,v 1.29 2004/05/16 06:21:02 seiichi Exp $
  */
 
 #include  "ml_term.h"
@@ -561,6 +561,64 @@ ml_term_get_cursor_line(
 	return  ml_screen_get_cursor_line( term->screen) ;
 }
 
+int
+ml_term_is_cursor_visible(
+	ml_term_t *  term
+	)
+{
+	return  ml_screen_is_cursor_visible( term->screen) ;
+}
+
+/*
+ * Not implemented yet.
+ */
+#if  0
+int
+ml_term_set_modified_region(
+	ml_term_t *  term ,
+	int  beg_char_index ,
+	int  beg_row ,
+	u_int  nchars ,
+	u_int  nrows
+	)
+{
+	return  0 ;
+}
+#endif
+
+/*
+ * Not used.
+ */
+#if  0
+int
+ml_term_set_modified_region_in_screen(
+	ml_term_t *  term ,
+	int  beg_char_index ,
+	int  beg_row ,
+	u_int  nchars ,
+	u_int  nrows
+	)
+{
+	int  row ;
+	ml_line_t *  line ;
+
+	ml_screen_logical( term->screen) ;
+
+	for( row = beg_row ; row < beg_row + nrows ; row ++)
+	{
+		if( ( line = ml_screen_get_line_in_screen( term->screen , row)))
+		{
+			ml_line_set_modified( line , beg_char_index , beg_char_index + nchars - 1) ;
+		}
+	}
+
+	/* ml_screen_render( term->screen) ; */
+	ml_screen_visual( term->screen) ;
+
+	return  1 ;
+}
+#endif
+
 int
 ml_term_set_modified_lines(
 	ml_term_t *  term ,
@@ -614,7 +672,7 @@ ml_term_set_modified_lines_in_screen(
 }
 
 int
-ml_term_set_modified_all(
+ml_term_set_modified_all_lines_in_screen(
 	ml_term_t *  term
 	)
 {
@@ -826,6 +884,14 @@ ml_term_enter_backscroll_mode(
 	ml_term_t *  term
 	)
 {
+	/* XXX */
+	if( term->vertical_mode)
+	{
+		kik_msg_printf( "Not supported backscrolling in vertical mode.\n") ;
+		
+		return  0 ;
+	}
+	
 	return  ml_enter_backscroll_mode( term->screen) ;
 }
 
@@ -891,7 +957,7 @@ ml_term_reverse_color(
 	)
 {
 	return  ml_screen_reverse_color( term->screen , beg_char_index , beg_row ,
-		end_char_index , end_row) ;
+			end_char_index , end_row) ;
 }
 
 int
@@ -904,7 +970,7 @@ ml_term_restore_color(
 	)
 {
 	return  ml_screen_restore_color( term->screen , beg_char_index , beg_row ,
-		end_char_index , end_row) ;
+			end_char_index , end_row) ;
 }
 
 u_int
