@@ -1,5 +1,5 @@
 /*
- *	$Id: ml_vt100_parser.c,v 1.35 2003/09/13 15:12:35 arakiken Exp $
+ *	$Id: ml_vt100_parser.c,v 1.40 2004/09/19 13:25:45 seiichi Exp $
  */
 
 #include  "ml_vt100_parser.h"
@@ -86,7 +86,7 @@ start_vt100_cmd(
 	{
 		ml_unuse_char_combining() ;
 	}
-	
+
 	if( vt100_parser->use_multi_col_char)
 	{
 		ml_use_multi_col_char() ;
@@ -166,7 +166,7 @@ receive_bytes(
 
 				p ++ ;
 			}
-			
+
 			if( ( path = kik_get_user_rc_path( path)) == NULL)
 			{
 				goto  end ;
@@ -176,7 +176,7 @@ receive_bytes(
 				open( path , O_CREAT | O_TRUNC | O_WRONLY , 0600)) == -1)
 			{
 				free( path) ;
-				
+
 				goto  end ;
 			}
 
@@ -206,7 +206,7 @@ end:
 			kik_msg_printf( "%c" , vt100_parser->seq[count]) ;
 		#endif
 		}
-	
+
 		kik_msg_printf( "[END]\n") ;
 	}
 #endif
@@ -238,7 +238,7 @@ flush_buffer(
 			char *  bytes ;
 
 			bytes = ml_char_bytes( &buffer->chars[count]) ;
-			
+
 			if( ml_char_size( &buffer->chars[count]) == 2)
 			{
 			#ifdef  DUMP_HEX
@@ -456,11 +456,40 @@ put_char(
 	}
 }
 
+static void
+save_cursor(
+	ml_vt100_parser_t *  vt100_parser
+	)
+{
+	vt100_parser->is_saved = 1 ;
+	vt100_parser->saved_fg_color = vt100_parser->fg_color ;
+	vt100_parser->saved_bg_color = vt100_parser->bg_color ;
+	vt100_parser->saved_is_bold = vt100_parser->is_bold ;
+	vt100_parser->saved_is_underlined = vt100_parser->is_underlined ;
+	vt100_parser->saved_is_reversed = vt100_parser->is_reversed ;
+
+	ml_screen_save_cursor( vt100_parser->screen) ;
+}
+
+static void
+restore_cursor(
+	ml_vt100_parser_t *  vt100_parser
+	)
+{
+	if( vt100_parser->is_saved){
+		vt100_parser->fg_color = vt100_parser->saved_fg_color ;
+		vt100_parser->bg_color = vt100_parser->saved_bg_color ;
+		vt100_parser->is_bold = vt100_parser->saved_is_bold ;
+		vt100_parser->is_underlined = vt100_parser->saved_is_underlined ;
+		vt100_parser->is_reversed = vt100_parser->saved_is_reversed ;
+	}
+	ml_screen_restore_cursor( vt100_parser->screen) ;
+}
 
 /*
  * VT100_PARSER Escape Sequence Commands.
  */
- 
+
 static void
 change_char_attr(
 	ml_vt100_parser_t *  vt100_parser ,
@@ -499,7 +528,7 @@ change_char_attr(
 	else if( flag == 7)
 	{
 		/* Inverse */
-		
+
 		vt100_parser->is_reversed = 1 ;
 	}
 	else if( flag == 22)
@@ -554,9 +583,7 @@ change_char_attr(
 	else if( flag == 39)
 	{
 		/* default fg */
-		
 		fg_color = ML_FG_COLOR ;
-		vt100_parser->is_reversed = 0 ;
 	}
 	else if( flag == 40)
 	{
@@ -593,7 +620,6 @@ change_char_attr(
 	else if( flag == 49)
 	{
 		bg_color = ML_BG_COLOR ;
-		vt100_parser->is_reversed = 0 ;
 	}
 #ifdef  DEBUG
 	else
@@ -716,7 +742,7 @@ inc_str_in_esc_seq(
 			{
 			#ifdef  DEBUG
 				kik_warn_printf( KIK_DEBUG_TAG " Ignored 0x%x inside escape sequences.\n" ,
-					**str_p) ; 
+					**str_p) ;
 			#endif
 			}
 		}
@@ -753,7 +779,7 @@ config_protocol_set(
 			{
 				break ;
 			}
-			
+
 			if( strcmp( key , "gen_proto_challenge") == 0)
 			{
 				ml_gen_proto_challenge() ;
@@ -763,7 +789,7 @@ config_protocol_set(
 				(*vt100_parser->config_listener->set)(
 					vt100_parser->config_listener->self , dev , key , val) ;
 			}
-			
+
 			/* XXX */
 			if( vt100_parser->config_listener == NULL)
 			{
@@ -817,7 +843,7 @@ config_protocol_save(
 		{
 			break ;
 		}
-		
+
 		/* XXX */
 		if( strcmp( key , "encoding") == 0)
 		{
@@ -834,6 +860,11 @@ config_protocol_save(
 	kik_conf_write_close( conf) ;
 	free( file) ;
 
+	if( HAS_CONFIG_LISTENER(vt100_parser,saved))
+	{
+		(*vt100_parser->config_listener->saved)( vt100_parser->config_listener->self) ;
+	}
+
 	return  1 ;
 }
 
@@ -864,7 +895,7 @@ config_protocol_get(
 		else if( ret == -1)
 		{
 			/* to_menu is necessarily 0, so it is pty that msg should be returned to. */
-			
+
 			char  msg[] = "#forbidden\n" ;
 
 			ml_write_to_pty( vt100_parser->pty , msg , sizeof( msg) - 1) ;
@@ -891,7 +922,7 @@ parse_vt100_escape_sequence(
 	if( vt100_parser->left == 0)
 	{
 		/* end of string */
-		
+
 		return  1 ;
 	}
 
@@ -937,18 +968,18 @@ parse_vt100_escape_sequence(
 				 *   col,row,fg_color,bg_color(in ml_cursor_t),
 				 * in other words , "char" is not saved , but maybe it works.
 				 *
-				 * BTW , owing to this behavior , 
+				 * BTW , owing to this behavior ,
 				 * "2. Test of screen features" - "Test of the SAVE/RESTORE CURSOR feature"
 				 * of vttest fails.
 				 */
 
-				ml_screen_save_cursor( vt100_parser->screen) ;
+				save_cursor( vt100_parser) ;
 			}
 			else if( *str_p == '8')
 			{
 				/* restore cursor */
-				
-				ml_screen_restore_cursor( vt100_parser->screen) ;
+
+				restore_cursor( vt100_parser) ;
 			}
 			else if( *str_p == '=')
 			{
@@ -983,7 +1014,7 @@ parse_vt100_escape_sequence(
 			else if( *str_p == 'E')
 			{
 				/* next line */
-				
+
 				ml_screen_line_feed( vt100_parser->screen) ;
 				ml_screen_goto_beg_of_line( vt100_parser->screen) ;
 			}
@@ -1073,14 +1104,22 @@ parse_vt100_escape_sequence(
 				num = 0 ;
 				while( num < 5)
 				{
+					while( '0' == *str_p)
+					{
+						if( inc_str_in_esc_seq( vt100_parser->screen ,
+									&str_p , &left , 0) == 0)
+						{
+							return  0 ;
+						}
+					}
 					if( '0' <= *str_p && *str_p <= '9')
 					{
-						u_char  digit[20] ;
+						u_char  digit[DIGIT_STR_LEN(int)] ;
 						int  count ;
 
 						digit[0] = *str_p ;
 
-						for( count = 1 ; count < 19 ; count ++)
+						for( count = 1 ; count < DIGIT_STR_LEN(int) -1 ; count ++)
 						{
 							if( inc_str_in_esc_seq( vt100_parser->screen ,
 								&str_p , &left , 0) == 0)
@@ -1868,11 +1907,11 @@ parse_vt100_escape_sequence(
 					}
 					else if( *str_p == 's')
 					{
-						ml_screen_save_cursor( vt100_parser->screen) ;
+						save_cursor( vt100_parser) ;
 					}
 					else if( *str_p == 'u')
 					{
-						ml_screen_restore_cursor( vt100_parser->screen) ;
+						restore_cursor( vt100_parser) ;
 					}
 					else if( *str_p == 'x')
 					{
@@ -2393,6 +2432,8 @@ ml_vt100_parser_new(
 		vt100_parser->col_size_of_east_asian_width_a = 1 ;
 	}
 
+	vt100_parser->is_saved = 0 ;
+
 	return  vt100_parser ;
 
 error:
