@@ -1,5 +1,5 @@
 /*
- *	$Id: ml_vt100_parser.h,v 1.7 2003/09/13 15:12:35 arakiken Exp $
+ *	$Id: ml_vt100_parser.h,v 1.9 2004/08/06 15:57:10 h_minami Exp $
  */
 
 #ifndef  __ML_VT100_PARSER_H__
@@ -60,6 +60,7 @@ typedef struct  ml_config_event_listener
 
 	void (*set)( void * , char * , char * , char *) ;
 	void (*get)( void * , char * , char * , int) ;
+	void (*saved)( void *) ;
 
 } ml_config_event_listener_t ;
 
@@ -108,6 +109,13 @@ typedef struct  ml_vt100_parser
 	int8_t  use_multi_col_char ;
 	int8_t  logging_vt_seq ;
 	
+	int is_saved;
+	ml_color_t  saved_fg_color ;
+	ml_color_t  saved_bg_color ;
+	int8_t  saved_is_bold ;
+	int8_t  saved_is_underlined ;
+	int8_t  saved_is_reversed ;
+
 } ml_vt100_parser_t ;
 
 
