@@ -1,5 +1,5 @@
 /*
- *	$Id: x_color.c,v 1.5 2003/06/11 11:01:10 h_minami Exp $
+ *	$Id: x_color.c,v 1.6 2003/12/27 09:06:22 arakiken Exp $
  */
 
 #include  "x_color.h"
@@ -19,7 +19,7 @@ alloc_closest_xcolor_pseudo(
 	unsigned short  red ,		/* 0 to 0xffff */
 	unsigned short  green ,		/* 0 to 0xffff */
 	unsigned short  blue ,		/* 0 to 0xffff */
-#ifdef ANTI_ALIAS
+#ifdef  USE_TYPE_XFT
 	XftColor *  ret_xcolor
 #else
 	XColor *  ret_xcolor
@@ -98,7 +98,7 @@ alloc_closest_xcolor_pseudo(
 		return 0 ;
 	}
 
-#ifdef ANTI_ALIAS
+#ifdef  USE_TYPE_XFT
 	ret_xcolor->pixel = closest_color.pixel ;
 	ret_xcolor->color.red = closest_color.red ;
 	ret_xcolor->color.green = closest_color.green ;
@@ -148,7 +148,7 @@ is_rgb_color_name(
 
 /* --- global functions --- */
 
-#ifdef  ANTI_ALIAS
+#ifdef  USE_TYPE_XFT
 
 int
 x_load_named_xcolor(
@@ -246,7 +246,7 @@ x_get_xcolor_rgb(
 	return  1 ;
 }
 
-#else
+#else	/* USE_TYPE_XFT */
 
 int
 x_load_named_xcolor(
