@@ -1,6 +1,6 @@
 /** @file
  *  @brief image handling functions using gdk-pixbuf
- *	$Id: x_imagelib_gdk.c,v 1.80 2003/07/07 05:26:06 h_minami Exp $
+ *	$Id: x_imagelib_gdk.c,v 1.85 2004/05/07 05:43:53 h_minami Exp $
  */
 
 #include <math.h>                        /* pow */
@@ -19,7 +19,7 @@
 #define SUCCESS 0
 
 /** Pixmap cache per display. */
-typedef struct  pixmap_cache_tag 
+typedef struct  pixmap_cache_tag
 {
 	Display *  display; /**<Display */
 	Pixmap  root;      /**<Root pixmap   !!! NOT owned by mlterm. DON'T FREE !!!*/
@@ -57,7 +57,7 @@ fetch_colormap(
 #endif
 		return  0 ;
 	}
-	for( i = 0 ; i < num_cells ; i ++)	
+	for( i = 0 ; i < num_cells ; i ++)
 		((*color_list)[i]).pixel = i ;
 
 	XQueryColors( display , cmap , *color_list, num_cells) ;
@@ -93,13 +93,21 @@ load_file(
 		/* free caches */
 		if( data)
 		{
+#if GDK_PIXBUF_MAJOR >= 2
+			g_object_unref( data) ;
+#else
 			gdk_pixbuf_unref( data) ;
+#endif /*GDK_PIXBUF_MAJOR*/
 			data = NULL ;
 		}
 		if( scaled)
 		{
+#if GDK_PIXBUF_MAJOR >= 2
+			g_object_unref( scaled) ;
+#else
 			gdk_pixbuf_unref( scaled) ;
-			data = NULL ;
+#endif /*GDK_PIXBUF_MAJOR*/
+			scaled = NULL ;
 		}
 		return  NULL ;
 	}
@@ -124,17 +132,29 @@ load_file(
 		name = strdup( path) ;
 
 		if( data)
+		{
+#if GDK_PIXBUF_MAJOR >= 2
+			g_object_unref( data) ;
+#else
 			gdk_pixbuf_unref( data) ;
+#endif /*GDK_PIXBUF_MAJOR*/
+		}
 
 		if( scaled) /* scaled one is not vaild now */
+		{
+#if GDK_PIXBUF_MAJOR >= 2
+			g_object_unref( scaled) ;
+#else
 			gdk_pixbuf_unref( scaled) ;
+#endif /*GDK_PIXBUF_MAJOR*/
+		}
 		scaled = NULL ;
 
-#ifndef OLD_GDK_PIXBUF
+#if GDK_PIXBUF_MAJOR >= 2
 		data = gdk_pixbuf_new_from_file( path, NULL) ;
 #else
 		data = gdk_pixbuf_new_from_file( path) ;
-#endif /*OLD_GDK_PIXBUF*/
+#endif /*GDK_PIXBUF_MAJOR*/
 
 		pixbuf = data ;
 
@@ -150,12 +170,12 @@ load_file(
 		height = gdk_pixbuf_get_height( data) ;
 
 	/* Old cached one became obsolete if width/height is changed */
-	if( ( width != gdk_pixbuf_get_width( data) ) ||
+	if( ( width != gdk_pixbuf_get_width( data)) ||
 	    ( height != gdk_pixbuf_get_height( data)))
 	{
 		if( scaled && gdk_pixbuf_get_width( scaled) == width && gdk_pixbuf_get_height( scaled) == height)
 		{
-#ifdef DEBUG
+#ifdef __DEBUG
 			kik_warn_printf(KIK_DEBUG_TAG "using the scaled pixbuf( %d x %d) from cache\n", width, height) ;
 #endif
 			pixbuf = scaled ;
@@ -163,8 +183,14 @@ load_file(
 		else
 		{
 			if( scaled)
+			{
+#if GDK_PIXBUF_MAJOR >= 2
+				g_object_unref( scaled) ;
+#else
 				gdk_pixbuf_unref( scaled) ;
-#ifdef DEBUG
+#endif /*GDK_PIXBUF_MAJOR*/
+			}
+#ifdef __DEBUG
 			kik_warn_printf(KIK_DEBUG_TAG "creating a scaled pixbuf(%d x %d) from %d %d \n", width, height) ;
 #endif
 			scaled = gdk_pixbuf_scale_simple( data, width, height,
@@ -174,7 +200,11 @@ load_file(
 	}
 	/* scaling ends here */
 
-	gdk_pixbuf_ref( pixbuf) ;
+#if GDK_PIXBUF_MAJOR >= 2
+		g_object_ref( pixbuf) ;
+#else
+		gdk_pixbuf_ref( pixbuf) ;
+#endif /*GDK_PIXBUF_MAJOR*/
 
 	return  pixbuf ;
 }
@@ -229,16 +259,19 @@ create_pixbuf_from_cardinals(
 	if( req_height == 0)
 		req_height = height ;
 
-	if( (req_width != width) || (req_height != height) )
+	if( (req_width != width) || (req_height != height))
 		scaled = gdk_pixbuf_scale_simple(pixbuf, req_width, req_height,
-						 GDK_INTERP_TILES);
+						 GDK_INTERP_TILES) ;
 	else
 		scaled = NULL ;
 
 	if( scaled)
 	{
-		gdk_pixbuf_unref( pixbuf);
-
+#if GDK_PIXBUF_MAJOR >= 2
+		g_object_unref( pixbuf) ;
+#else
+		gdk_pixbuf_unref( pixbuf) ;
+#endif /*GDK_PIXBUF_MAJOR*/
 		return  scaled ;
 	}
 	else
@@ -261,14 +294,15 @@ create_cardinals_from_bixbuf(
 	unsigned char *pixel ;
 	int i, j ;
 
-	/* create (maybe shriked) copy */ 
+	*cardinal = malloc( (width * height + 2) *4) ;
+	if( !(*cardinal))
+		return  -1 ;
+
+	/* create (maybe shriked) copy */
 	pixbuf = gdk_pixbuf_scale_simple(pixbuf, width, height, GDK_INTERP_TILES) ;
 
 	rowstride = gdk_pixbuf_get_rowstride( pixbuf) ;
 	line = gdk_pixbuf_get_pixels( pixbuf) ;
-	*cardinal = malloc( (width * height + 2) *4) ;
-	if( !(*cardinal))
-		return  -1 ;
 
 	/* format of the array is {width, height, ARGB[][]} */
 	(*cardinal)[0] = width ;
@@ -278,7 +312,7 @@ create_cardinals_from_bixbuf(
 			pixel = line ;
 			line += rowstride;
 			for( j = 0; j < width; j++) {
-				/* from RGBA to ARGB conversion */
+				/* RGBA to ARGB */
 				(*cardinal)[(i*width+j)+2] = ((((((u_int32_t)(pixel[3]) << 8)
 								 + pixel[0]) << 8)
 							       + pixel[1]) << 8) + pixel[2] ;
@@ -294,7 +328,7 @@ create_cardinals_from_bixbuf(
 			line += rowstride;
 			for (j = 0; j < width; j++) {
 				/* all pixels are completely opaque (0xFF) */
-				(*cardinal)[(i*width+j)+2] = ((((((u_int32_t)(0x0000FF) <<8 )
+				(*cardinal)[(i*width+j)+2] = ((((((u_int32_t)(0x0000FF) <<8)
 								 + pixel[0]) << 8)
 							       + pixel[1]) << 8) + pixel[2] ;
 				pixel += 3 ;
@@ -302,11 +336,15 @@ create_cardinals_from_bixbuf(
 		}
 	}
 
+#if GDK_PIXBUF_MAJOR >= 2
+	g_object_unref( pixbuf) ;
+#else
 	gdk_pixbuf_unref( pixbuf) ;
+#endif /*GDK_PIXBUF_MAJOR*/
 
 	return  SUCCESS ;
 }
-/* seek the closet color */
+/* seek the closest color */
 static int
 closest_color_index(
 	Display *  display ,
@@ -400,7 +438,7 @@ pixbuf_to_pixmap_pseudocolor(
 	line = gdk_pixbuf_get_pixels( pixbuf) ;
 	gc = XCreateGC (display, pixmap, 0, &gcv) ;
 
-	for ( y = 0 ; y < height ; y++ )
+	for ( y = 0 ; y < height ; y++)
 	{
 		pixel = line ;
 #ifdef USE_FS
@@ -408,9 +446,9 @@ pixbuf_to_pixmap_pseudocolor(
 					       pixel[0] - diff_cur[0],
 					       pixel[1] - diff_cur[1],
 					       pixel[2] - diff_cur[2]) ;
-		diff_r = (color_list[closest].red   >>8 ) - pixel[0];
-		diff_g = (color_list[closest].green >>8 ) - pixel[1];
-		diff_b = (color_list[closest].blue  >>8 ) - pixel[2];
+		diff_r = (color_list[closest].red   >>8) - pixel[0];
+		diff_g = (color_list[closest].green >>8) - pixel[1];
+		diff_b = (color_list[closest].blue  >>8) - pixel[2];
 
 		diff_cur[3*1 + 0 ] += diff_r /2;
 		diff_cur[3*1 + 1 ] += diff_g /2;
@@ -435,16 +473,16 @@ pixbuf_to_pixmap_pseudocolor(
 		XDrawPoint( display, pixmap, gc, 0, y) ;
 		pixel += bytes_per_pixel ;
 
-		for ( x = 1 ; x < width -2 ; x++ )
+		for ( x = 1 ; x < width -2 ; x++)
 		{
 #ifdef USE_FS
 			closest = closest_color_index( display, screen, color_list, num_cells,
 						       pixel[0] - diff_cur[3*x +0],
 						       pixel[1] - diff_cur[3*x +1],
 						       pixel[2] - diff_cur[3*x +2]) ;
-			diff_r = (color_list[closest].red   >>8 ) - pixel[0];
-			diff_g = (color_list[closest].green >>8 ) - pixel[1];
-			diff_b = (color_list[closest].blue  >>8 ) - pixel[2];
+			diff_r = (color_list[closest].red   >>8) - pixel[0];
+			diff_g = (color_list[closest].green >>8) - pixel[1];
+			diff_b = (color_list[closest].blue  >>8) - pixel[2];
 
 			diff_cur[3*(x+1) + 0 ] += diff_r /2;
 			diff_cur[3*(x+1) + 1 ] += diff_g /2;
@@ -465,7 +503,7 @@ pixbuf_to_pixmap_pseudocolor(
 			closest = closest_color_index( display, screen, color_list, num_cells,
 						       pixel[0] ,
 						       pixel[1] ,
-						       pixel[2] ) ;
+						       pixel[2]) ;
 #endif /* USE_SF */
 
 			XSetForeground( display, gc, closest) ;
@@ -478,9 +516,9 @@ pixbuf_to_pixmap_pseudocolor(
 					       pixel[0] - diff_cur[3*x +0],
 					       pixel[1] - diff_cur[3*x +1],
 					       pixel[2] - diff_cur[3*x +2]) ;
-		diff_r = (color_list[closest].red   >>8 ) - pixel[0];
-		diff_g = (color_list[closest].green >>8 ) - pixel[1];
-		diff_b = (color_list[closest].blue  >>8 ) - pixel[2];
+		diff_r = (color_list[closest].red   >>8) - pixel[0];
+		diff_g = (color_list[closest].green >>8) - pixel[1];
+		diff_b = (color_list[closest].blue  >>8) - pixel[2];
 
 		diff_next[3*(x-1) +0] += diff_r /4;
 		diff_next[3*(x-1) +1] += diff_g /4;
@@ -489,9 +527,9 @@ pixbuf_to_pixmap_pseudocolor(
 		diff_next[3*(x+0) +0] += diff_r /4;
 		diff_next[3*(x+0) +1] += diff_g /4;
 		diff_next[3*(x+0) +2] += diff_b /4;
+
 		temp = diff_cur;
 		diff_cur = diff_next;
-		/* next line is overwritten and don't need a special initialization */
 		diff_next = temp;
 #else
 		closest = closest_color_index( display, screen, color_list, num_cells,
@@ -539,7 +577,7 @@ tile_pixmap(
 
 	if ( force_copy ||
 	     aw < DisplayWidth( display, screen) ||
-	     ah < DisplayHeight( display, screen) )
+	     ah < DisplayHeight( display, screen))
 	{
 		/* Some WM needs tiling... sigh.*/
 		result = XCreatePixmap( display, pixmap,
@@ -553,6 +591,7 @@ tile_pixmap(
 		XFillRectangle( display, result, gc, 0, 0,
 				DisplayWidth( display, screen),
 				DisplayHeight( display, screen)) ;
+
 		return  result ;
 	}
 
@@ -574,9 +613,8 @@ root_pixmap(
 	if( !id)
 	{
 #ifdef DEBUG
-			kik_warn_printf(KIK_DEBUG_TAG "_XROOTPMAP_ID is not available\n") ;
+		kik_warn_printf(KIK_DEBUG_TAG "_XROOTPMAP_ID is not available\n") ;
 #endif
-
 		return  None ;
 	}
 
@@ -589,17 +627,14 @@ root_pixmap(
 			Pixmap root ;
 			root = *((Drawable *)prop) ;
 			XFree( prop) ;
-#ifdef DEBUG
-			kik_warn_printf(KIK_DEBUG_TAG "pixmap id # %d is read\n" , root) ;
-#endif
 
 			return  root ;
 		}
 #ifdef DEBUG
 		else
 		{
-			kik_warn_printf(KIK_DEBUG_TAG "failed tp read prop\n") ;		}
-
+			kik_warn_printf(KIK_DEBUG_TAG "failed tp read prop\n") ;
+		}
 #endif
 	}
 
@@ -698,10 +733,10 @@ is_picmod_eq(
 	x_picture_modifier_t *  b
 	)
 {
-	if( a == b )
+	if( a == b)
 		return  1 ;
 
-	if ( a == NULL )
+	if ( a == NULL)
 	{
 		a = b ;
 		b = NULL ;
@@ -833,10 +868,10 @@ pixbuf_to_ximage_truecolor(
 			pixel = line ;
 			for (j = 0; j < width; j++)
 			{
-				XPutPixel( image, j, i, 
+				XPutPixel( image, j, i,
 					   ( ( (pixel[0] >> r_limit) << r_offset) & r_mask) |
 					   ( ( (pixel[1] >> g_limit) << g_offset) & g_mask) |
-					   ( ( (pixel[2] >> b_limit) << b_offset) & b_mask) ) ;
+					   ( ( (pixel[2] >> b_limit) << b_offset) & b_mask)) ;
 				pixel += bytes_per_pixel ;
 			}
 			line += rowstride ;
@@ -974,26 +1009,26 @@ compose_truecolor(
 
 	rowstride = gdk_pixbuf_get_rowstride( pixbuf) ;
 	line = gdk_pixbuf_get_pixels( pixbuf) ;
-	
+
 	for( i = 0; i < height; i++)
 	{
 		pixel = line ;
 		for( j = 0; j < width; j++)
 		{
 			data = XGetPixel( image, j, i) ;
-			
+
 			r = ((data & r_mask) >>r_offset) ;
 			g = ((data & g_mask) >>g_offset) ;
 			b = ((data & b_mask) >>b_offset) ;
-			
+
 			r = (r*(256 - pixel[3]) + pixel[0] *  pixel[3])>>8 ;
 			g = (g*(256 - pixel[3]) + pixel[1] *  pixel[3])>>8 ;
 			b = (b*(256 - pixel[3]) + pixel[2] *  pixel[3])>>8 ;
-			
+
 			XPutPixel( image, j, i,
-				   (r <<r_offset ) |
-				   (g <<g_offset ) |
-				   (b <<b_offset ) ) ;
+				   (r <<r_offset) |
+				   (g <<g_offset) |
+				   (b <<b_offset)) ;
 			pixel += 4 ;
 		}
 		line += rowstride ;
@@ -1018,7 +1053,7 @@ compose_pseudocolor(
 	unsigned int width, height, rowstride ;
 	unsigned char *line ;
 	unsigned char *pixel ;
-	long  data ; 
+	long  data ;
 	XColor *  color_list ;
 
 	num_cells = fetch_colormap( display, screen, &color_list) ;
@@ -1043,15 +1078,15 @@ compose_pseudocolor(
 			r = color_list[data].red >>8 ;
 			g = color_list[data].green >>8 ;
 			b = color_list[data].blue >>8 ;
-			
+
 			r = (r*(256 - pixel[3]) + pixel[0] *  pixel[3])>>8 ;
 			g = (g*(256 - pixel[3]) + pixel[1] *  pixel[3])>>8 ;
 			b = (b*(256 - pixel[3]) + pixel[2] *  pixel[3])>>8 ;
-			
+
 			XPutPixel( image, j, i,
 				   closest_color_index( display, screen,
 							color_list, num_cells,
-							r, g, b ) );
+							r, g, b)) ;
 			pixel += 4 ;
 		}
 		line += rowstride ;
@@ -1148,7 +1183,7 @@ pixbuf_to_pixmap_and_mask(
 
 		*mask = XCreatePixmap( display,
 				       DefaultRootWindow( display),
-				       width, height, 1) ;		
+				       width, height, 1) ;
 		gc = XCreateGC (display, *mask, 0, &gcv) ;
 
 		XSetForeground( display, gc, 0) ;
@@ -1168,7 +1203,7 @@ pixbuf_to_pixmap_and_mask(
 					XDrawPoint( display, *mask, gc, j, i) ;
 				pixel += 4 ;
 			}
-       			line += rowstride ;
+			line += rowstride ;
 		}
 		XFreeGC( display, gc) ;
 	}
@@ -1191,19 +1226,20 @@ value_table_refresh(
 	real_gamma = (double)(mod->gamma) / 100 ;
 	real_contrast = (double)(mod->contrast) / 100 ;
 	real_brightness = (double)(mod->brightness) / 100 ;
-	
+
 	for( i = 0 ; i < 256 ; i++)
 	{
-		tmp = real_contrast * (255 * pow(((double)i + 0.5)/ 255, real_gamma) -128 ) + 128 *  real_brightness ;
+		tmp = real_contrast * (255 * pow(((double)i + 0.5)/ 255, real_gamma) -128)
+			+ 128 *  real_brightness ;
 		if( tmp >= 255)
 			break;
 
-		if( tmp < 0 )			
+		if( tmp < 0)
 			value_table[i] = 0 ;
 		else
 			value_table[i] = tmp ;
 	}
-	for( ; i< 256 ; i++ )
+	for( ; i< 256 ; i++)
 		value_table[i] = 255 ;
 }
 
@@ -1218,7 +1254,7 @@ modify_image(
 	unsigned char *line ;
 	unsigned char *pixel ;
 
-	if ( !pixbuf )
+	if ( !pixbuf)
 		return  -1 ;
 	bytes_per_pixel = (gdk_pixbuf_get_has_alpha( pixbuf)) ? 4:3 ;
 	width = gdk_pixbuf_get_width (pixbuf) ;
@@ -1230,7 +1266,7 @@ modify_image(
 	if( is_picmod_eq( pic_mod, NULL))
 		return  SUCCESS ;
 
-       	line = gdk_pixbuf_get_pixels( pixbuf) ;
+	line = gdk_pixbuf_get_pixels( pixbuf) ;
 
 	value_table_refresh( pic_mod) ;
 
@@ -1273,7 +1309,7 @@ modify_pixmap(
 	XVisualInfo *  vinfolist ;
 	XVisualInfo  vinfo ;
 
-	if ( !pixmap )
+	if ( !pixmap)
 		return  -1 ;
 
 	if( is_picmod_eq( pic_mod, NULL))
@@ -1299,36 +1335,36 @@ modify_pixmap(
 	{
 		unsigned char  r, g, b ;
 		int r_limit, g_limit, b_limit ;
-		long  data;			
-		
+		long  data ;
+
 		r_mask = vinfolist[0].red_mask ;
 		g_mask = vinfolist[0].green_mask ;
 		b_mask = vinfolist[0].blue_mask ;
-		
+
 		r_offset = lsb( r_mask) ;
 		g_offset = lsb( g_mask) ;
-		b_offset = lsb( b_mask) ;		
+		b_offset = lsb( b_mask) ;
 
 		r_limit = 8 + r_offset - msb( r_mask) ;
 		g_limit = 8 + g_offset - msb( g_mask) ;
 		b_limit = 8 + b_offset - msb( b_mask) ;
-		
+
 		value_table_refresh( pic_mod) ;
 		for (i = 0; i < height; i++)
 		{
 			for (j = 0; j < width; j++)
 			{
 				data = XGetPixel( image, j, i) ;
-				
+
 				r = ((data & r_mask) >> r_offset)<<r_limit ;
 				g = ((data & g_mask) >> g_offset)<<g_limit ;
 				b = ((data & b_mask) >> b_offset)<<b_limit ;
-				
+
 				r = value_table[r] ;
 				g = value_table[g] ;
 				b = value_table[b] ;
-				
-				XPutPixel( image, j, i, 
+
+				XPutPixel( image, j, i,
 					   (r >> r_limit) << r_offset |
 					   (g >> g_limit) << g_offset |
 					   (b >> b_limit) << b_offset) ;
@@ -1357,7 +1393,7 @@ modify_pixmap(
 				for (j = 0; j < width; j++)
 				{
 					data = XGetPixel( image, j, i) ;
-					
+
 					r = (color_list[data].red >>8)  ;
 					g = (color_list[data].green >>8)  ;
 					b = (color_list[data].blue >>8)  ;
@@ -1366,10 +1402,10 @@ modify_pixmap(
 					g = value_table[g] ;
 					b = value_table[b] ;
 
-					XPutPixel( image, j, i, 
+					XPutPixel( image, j, i,
 						   closest_color_index( display, screen,
 									color_list, num_cells,
-									r, g, b ) );
+									r, g, b)) ;
 				}
 
 			}
@@ -1396,9 +1432,9 @@ x_imagelib_display_opened(
 {
 	if (display_count == 0)
 	{
-#ifndef OLD_GDK_PIXBUF
+#if GDK_PIXBUF_MAJOR >= 2
 		g_type_init() ;
-#endif /*OLD_GDK_PIXBUF*/
+#endif /*GDK_PIXBUF_MAJOR*/
 	}
 	display_count ++ ;
 	return  1 ;
@@ -1449,7 +1485,7 @@ x_imagelib_load_file_for_background(
 	    && (is_picmod_eq( pic_mod, cached_mod))
 	    && ACTUAL_WIDTH(win) == gdk_pixbuf_get_width( cached_pixbuf)
 	    && ACTUAL_HEIGHT(win) == gdk_pixbuf_get_height( cached_pixbuf)
-	    && (strcmp( file_path, cached_pixbuf_file_path) == 0) )
+	    && (strcmp( file_path, cached_pixbuf_file_path) == 0))
 	{
 		/* use pixbuf in the cache */
 		pixbuf = cached_pixbuf ;
@@ -1465,27 +1501,37 @@ x_imagelib_load_file_for_background(
 		}
 
 		if( cached_pixbuf)
+		{
+#if GDK_PIXBUF_MAJOR >= 2
+			g_object_unref( cached_pixbuf) ;
+#else
 			gdk_pixbuf_unref( cached_pixbuf) ;
+#endif /*GDK_PIXBUF_MAJOR*/
+		}
 		if( cached_mod)
 			free(cached_mod) ;
 		if( cached_pixbuf_file_path)
-                        free(cached_pixbuf_file_path) ;
+			free(cached_pixbuf_file_path) ;
 		cached_pixbuf_file_path = strdup( file_path) ;
 
-		if( is_picmod_eq( pic_mod, NULL) )
+		if( is_picmod_eq( pic_mod, NULL))
 		{
 			cached_pixbuf = pixbuf ;
 		}
 		else
 		{
 			cached_pixbuf = gdk_pixbuf_copy(pixbuf) ;
+#if GDK_PIXBUF_MAJOR >= 2
+			g_object_unref( pixbuf) ;
+#else
 			gdk_pixbuf_unref( pixbuf) ;
+#endif /*GDK_PIXBUF_MAJOR*/
 			modify_image( cached_pixbuf, pic_mod) ;
 		}
 
 		if( pic_mod)
 		{
-			cached_mod = malloc(sizeof(x_picture_modifier_t)) ;		
+			cached_mod = malloc(sizeof(x_picture_modifier_t)) ;
 			memcpy( cached_mod, pic_mod, sizeof(x_picture_modifier_t)) ;
 		}
 		else
@@ -1496,7 +1542,7 @@ x_imagelib_load_file_for_background(
 		pixbuf = cached_pixbuf ;
 	}
 
-	if( gdk_pixbuf_get_has_alpha ( pixbuf) )
+	if( gdk_pixbuf_get_has_alpha ( pixbuf))
 	{
 		pixmap = x_imagelib_get_transparent_background( win, NULL) ;
 		if( compose_to_pixmap( win->display, win->screen,
@@ -1595,7 +1641,7 @@ x_imagelib_get_transparent_background(
 	pixmap = XCreatePixmap( win->display, win->my_window, ACTUAL_WIDTH(win), ACTUAL_HEIGHT(win),
 				DefaultDepth( win->display, win->screen)) ;
 
-	gc = XCreateGC( win->display, win->my_window, 0, 0 ) ;
+	gc = XCreateGC( win->display, win->my_window, 0, 0) ;
 
 	if ( !is_picmod_eq( cache->pic_mod, pic_mod))
 	{
@@ -1610,7 +1656,7 @@ x_imagelib_get_transparent_background(
 						     win->screen,
 						     gc,
 						     current_root
-						     ,1 ) ;
+						     ,1) ;
 			if(cache->pic_mod)
 				free(cache->pic_mod) ;
 			cache->pic_mod = malloc(sizeof(x_picture_modifier_t)) ;
@@ -1623,23 +1669,23 @@ x_imagelib_get_transparent_background(
 						     win->screen,
 						     gc,
 						     current_root,
-						     0 ) ;
+						     0) ;
 		}
 	}
 	else
 	{
-		if( cache->cooked == None )
+		if( cache->cooked == None)
 		{
 			cache->cooked = tile_pixmap( win->display,
 						     win->screen,
 						     gc,
 						     current_root,
-						     0 ) ;
+						     0) ;
 		}
 	}
 
 	XCopyArea( win->display, cache->cooked, pixmap, gc, x, y, width, height, pix_x, pix_y) ;
-	XFreeGC( win->display, gc ) ;
+	XFreeGC( win->display, gc) ;
 
 	return  pixmap ;
 }
@@ -1650,7 +1696,7 @@ x_imagelib_get_transparent_background(
  *\param cardinal Returns pointer to a data structure for the extended WM hint spec.
  *\param pixmap Returns an image pixmap for the old WM hint.
  *\param mask Returns a mask bitmap for the old WM hint.
- *\param width Pointer to the desired width. If *width is 0, the returned image would not be scaled and *width would be overwritten by its width. "width" can be NULL and the image would not be scaled and nothing would be returned in this case.   
+ *\param width Pointer to the desired width. If *width is 0, the returned image would not be scaled and *width would be overwritten by its width. "width" can be NULL and the image would not be scaled and nothing would be returned in this case.
  *\param height Pointer to the desired height. *height can be 0 and height can be NULL(see "width" 's description)
  *
  *\return  Success => 1, Failure => 0
@@ -1683,7 +1729,7 @@ int x_imagelib_load_file(
 	{
 		dst_height = *height ;
 	}
-	
+
 	if( path)
 	{
 		/* create a pixbuf from the file and create a cardinal array */
@@ -1707,10 +1753,10 @@ int x_imagelib_load_file(
 			return  0 ;
 
 	}
-	
+
 	dst_width = gdk_pixbuf_get_width( pixbuf) ;
 	dst_height = gdk_pixbuf_get_height( pixbuf) ;
-	
+
 	/* Create the Icon pixmap & mask to be used in WMHints.
 	 * Note that none as a result is acceptable.
 	 * Pixmaps can't be cached since the last pixmap may be freed by someone... */
@@ -1726,7 +1772,11 @@ int x_imagelib_load_file(
 						       DefaultScreen( display),
 						       pixbuf, pixmap, mask) != SUCCESS)
 			{
+#if GDK_PIXBUF_MAJOR >= 2
+				g_object_unref( pixbuf) ;
+#else
 				gdk_pixbuf_unref( pixbuf) ;
+#endif /*GDK_PIXBUF_MAJOR*/
 				XFreePixmap( display, *pixmap) ;
 				*pixmap = None ;
 				XFreePixmap( display, *mask) ;
@@ -1741,7 +1791,11 @@ int x_imagelib_load_file(
 			if( pixbuf_to_pixmap( display, DefaultScreen( display),
 					      pixbuf, *pixmap) != SUCCESS)
 			{
+#if GDK_PIXBUF_MAJOR >= 2
+				g_object_unref( pixbuf) ;
+#else
 				gdk_pixbuf_unref( pixbuf) ;
+#endif /*GDK_PIXBUF_MAJOR*/
 				XFreePixmap( display, *pixmap) ;
 				*pixmap = None ;
 
@@ -1759,7 +1813,10 @@ int x_imagelib_load_file(
 		*height = dst_height ;
 	}
 
+#if GDK_PIXBUF_MAJOR >= 2
+	g_object_unref( pixbuf) ;
+#else
 	gdk_pixbuf_unref( pixbuf) ;
-
+#endif /*GDK_PIXBUF_MAJOR*/
 	return  1 ;
 }
