@@ -1,5 +1,5 @@
 /*
- *	$Id: x_shortcut.c,v 1.7 2003/06/13 12:27:49 arakiken Exp $
+ *	$Id: x_shortcut.c,v 1.11 2004/10/10 12:48:44 seiichi Exp $
  */
 
 #include  "x_shortcut.h"
@@ -32,9 +32,8 @@ typedef struct  key_func_table
 
 static key_func_table_t  key_func_table[] =
 {
-	{ "XIM_OPEN" , XIM_OPEN , } ,
-	{ "XIM_CLOSE" , XIM_CLOSE , } ,
-	{ "EXT_KBD" , EXT_KBD } ,
+	{ "IM_HOTKEY" , IM_HOTKEY , } ,
+	{ "EXT_KBD" , EXT_KBD , } ,
 	{ "OPEN_SCREEN" , OPEN_SCREEN , } ,
 	{ "OPEN_PTY" , OPEN_PTY , } ,
 	{ "NEXT_PTY" , NEXT_PTY , } ,
@@ -98,10 +97,24 @@ parse(
 		return  0 ;
 	}
 
+	for( count = 0 ; count < sizeof( key_func_table) / sizeof( key_func_table_t) ; count ++)
+	{
+		x_key_t *  map_entry ;
+
+		map_entry = shortcut->map + key_func_table[count].func ;
+		if( (map_entry->ksym == ksym) &&
+		    (map_entry->state = state))
+		{
+			map_entry->is_used = 0 ;
+		}
+	}
+
+
 	if( *oper == '"')
 	{
 		char *  str ;
 		char *  p ;
+		x_str_key_t *  str_map ;
 
 		if( ( str = malloc( strlen( oper))) == NULL)
 		{
@@ -127,7 +140,7 @@ parse(
 				if( *oper == '\\')
 				{
 					oper ++ ;
-					
+
 					if( *oper == '\0')
 					{
 						break ;
@@ -166,22 +179,26 @@ parse(
 
 		*p = '\0' ;
 
-		if( ( shortcut->str_map = realloc( shortcut->str_map ,
-				sizeof( x_str_key_t) * (shortcut->str_map_size + 1))) == NULL)
+		str_map = realloc( shortcut->str_map ,
+				   sizeof( x_str_key_t) * (shortcut->str_map_size + 1)) ;
+
+		if( str_map == NULL)
 		{
 			free( str) ;
 
 			return  0 ;
 		}
 
-		shortcut->str_map[shortcut->str_map_size].ksym = ksym ;
-		shortcut->str_map[shortcut->str_map_size].state = state ;
-		shortcut->str_map[shortcut->str_map_size].str = str ;
+		str_map[shortcut->str_map_size].ksym = ksym ;
+		str_map[shortcut->str_map_size].state = state ;
+		str_map[shortcut->str_map_size].str = str ;
+
 		shortcut->str_map_size ++ ;
+		shortcut->str_map = str_map ;
 
 		return  1 ;
 	}
-	
+
 	for( count = 0 ; count < sizeof( key_func_table) / sizeof( key_func_table_t) ; count ++)
 	{
 		if( strcmp( oper , key_func_table[count].name) == 0)
@@ -215,13 +232,10 @@ x_shortcut_init(
 {
 	x_key_t  default_key_map[] =
 	{
-		/* XIM_OPEN */
-		{ XK_space , ShiftMask , 1 , } ,
-
-		/* XIM_CLOSE(not used) */
+		/* IM_HOTKEY */
 		{ 0 , 0 , 0 , } ,
 
-		/* EXT_KBD(not used) */
+		/* EXT_KBD(obsolete) */
 		{ 0 , 0 , 0 , } ,
 		
 		/* OPEN_SCREEN */
@@ -281,7 +295,7 @@ x_shortcut_final(
 }
 
 int
-x_shortcut_read_conf(
+x_read_shortcut_config(
 	x_shortcut_t *  shortcut ,
 	char *  filename
 	)
@@ -301,6 +315,17 @@ x_shortcut_read_conf(
 
 	while( kik_conf_io_read( from , &key , &value))
 	{
+		/*
+		 * XIM_OPEN and XIM_CLOSE are removed.
+		 */
+		if( strcmp( value, "XIM_OPEN") == 0 ||
+		    strcmp( value, "XIM_CLOSE") == 0)
+		{
+			/* This warning will be removed. */
+			kik_warn_printf( "%s in %s is no longer supported\n",
+					 value , filename);
+		}
+
 		/*
 		 * [shortcut key]=[operation]
 		 */
