@@ -1,9 +1,9 @@
 /*
- *	$Id: x_shortcut.h,v 1.4 2003/03/01 16:39:38 arakiken Exp $
+ *	$Id: x_shortcut.h,v 1.8 2004/10/20 18:46:53 seiichi Exp $
  */
 
 /*
- * this manages short-cut keys of x_screen key events.
+ * This manages short-cut keys of x_screen key events.
  */
  
 #ifndef  __X_SHORTCUT_H__
@@ -16,8 +16,7 @@
 
 typedef enum  x_key_func
 {
-	XIM_OPEN ,
-	XIM_CLOSE ,
+	IM_HOTKEY ,
 	EXT_KBD ,
 	OPEN_SCREEN ,
 	OPEN_PTY ,
@@ -63,7 +62,7 @@ int  x_shortcut_init( x_shortcut_t *  shortcut) ;
 
 int  x_shortcut_final( x_shortcut_t *  shortcut) ;
 
-int  x_shortcut_read_conf( x_shortcut_t *  shortcut , char *  filename) ;
+int  x_read_shortcut_config( x_shortcut_t *  shortcut , char *  filename) ;
 
 int  x_shortcut_match( x_shortcut_t *  shortcut , x_key_func_t  func , KeySym  sym , u_int  state) ;
 
