@@ -1,5 +1,5 @@
 /*
- *	$Id: x_termcap.h,v 1.3 2002/07/03 04:16:33 h_minami Exp $
+ *	$Id: x_termcap.h,v 1.4 2003/10/11 12:23:05 arakiken Exp $
  */
 
 #ifndef  __X_TERMCAP_H__
@@ -49,7 +49,7 @@ int  x_termcap_init( x_termcap_t *  termcap) ;
 
 int  x_termcap_final( x_termcap_t *  termcap) ;
 
-int  x_termcap_read_conf( x_termcap_t *  termcap , char *  filename) ;
+int  x_read_termcap_config( x_termcap_t *  termcap , char *  filename) ;
 
 x_termcap_entry_t *  x_termcap_get_entry( x_termcap_t *  termcap , char *  name) ;
 
