@@ -1,13 +1,13 @@
 /*
- *	$Id: x_window.c,v 1.56 2003/10/05 13:08:28 arakiken Exp $
+ *	$Id: x_window.c,v 1.72 2004/10/13 10:59:41 seiichi Exp $
  */
 
 /*
- * Functions designed and implemented by Minami Hirokazu(minami@chem.s.u-tokyo.ac.jp) are:
+ * Functions designed and implemented by Minami Hirokazu(minami@mistfall.net) are:
  * - XDND support
  * - Extended Window Manager Hint(Icon) support
  */
- 
+
 #include  "x_window.h"
 
 #include  <stdlib.h>		/* abs */
@@ -20,8 +20,10 @@
 #include  "x_xic.h"
 #include  "x_window_manager.h"
 #include  "x_imagelib.h"
-#include  "x_dnd.h"
 
+#ifndef  DISABLE_XDND
+#include  "x_dnd.h"
+#endif
 
 /*
  * Atom macros.
@@ -29,10 +31,15 @@
  */
 
 #define  XA_COMPOUND_TEXT(display)  (XInternAtom(display , "COMPOUND_TEXT" , False))
+#define  XA_TARGETS(display)  (XInternAtom(display , "TARGETS" , False))
+#ifdef  DEBUG
+#define  XA_MULTIPLE(display)  (XInternAtom(display , "MULTIPLE" , False))
+#endif
 #define  XA_TEXT(display)  (XInternAtom( display , "TEXT" , False))
 #define  XA_UTF8_STRING(display)  (XInternAtom(display , "UTF8_STRING" , False))
 #define  XA_SELECTION(display) (XInternAtom(display , "MLTERM_SELECTION" , False))
 #define  XA_DELETE_WINDOW(display) (XInternAtom(display , "WM_DELETE_WINDOW" , False))
+#define  XA_TAKE_FOCUS(display) (XInternAtom(display , "WM_TAKE_FOCUS" , False))
 #define  XA_INCR(display) (XInternAtom(display, "INCR", False))
 #define  XA_XROOTPMAP_ID(display) (XInternAtom(display, "_XROOTPMAP_ID", False))
 
@@ -41,6 +48,24 @@
  */
 #define  XA_NET_WM_ICON(display) (XInternAtom(display, "_NET_WM_ICON", False))
 
+
+/*
+ * Motif Window Manager Hint (for borderless window)
+ */
+#define  XA_MWM_INFO(display) (XInternAtom(display, "_MOTIF_WM_INFO", True))
+#define  XA_MWM_HINTS(display) (XInternAtom(display, "_MOTIF_WM_HINTS", True))
+
+typedef struct {
+	u_int32_t  flags ;
+	u_int32_t  functions ;
+	u_int32_t  decorations ;
+	int32_t  inputMode ;
+	u_int32_t  status ;
+} MWMHints_t ;
+
+#define MWM_HINTS_DECORATIONS   (1L << 1)
+
+
 #define  MAX_CLICK  3			/* max is triple click */
 
 #if  0
@@ -109,7 +134,7 @@ update_pic_transparent(
 	{
 		return  0 ;
 	}
-	
+
 	if( ! x_picture_load_background( &pic))
 	{
 		x_picture_final( &pic) ;
@@ -120,7 +145,7 @@ update_pic_transparent(
 	set_transparent( win , pic.pixmap) ;
 
 	x_picture_final( &pic) ;
-	
+
 	return  1 ;
 }
 
@@ -155,7 +180,7 @@ notify_focus_out_to_children(
 	{
 		(*win->window_unfocused)( win) ;
 	}
-	
+
 	x_xic_unset_focus( win) ;
 
 	for( count = 0 ; count < win->num_of_children ; count ++)
@@ -233,7 +258,7 @@ is_descendant_window(
 	)
 {
 	int  count ;
-	
+
 	if( win->my_window == window)
 	{
 		return  1 ;
@@ -268,7 +293,7 @@ total_min_width(
 	u_int  min_width ;
 
 	min_width = win->min_width + win->margin * 2 ;
-	
+
 	for( count = 0 ; count < win->num_of_children ; count ++)
 	{
 		if( win->children[count]->is_mapped)
@@ -289,7 +314,7 @@ total_min_height(
 	u_int  min_height ;
 
 	min_height = win->min_height + win->margin * 2 ;
-	
+
 	for( count = 0 ; count < win->num_of_children ; count ++)
 	{
 		if( win->children[count]->is_mapped)
@@ -310,7 +335,7 @@ total_base_width(
 	u_int  base_width ;
 
 	base_width = win->base_width + win->margin * 2 ;
-	
+
 	for( count = 0 ; count < win->num_of_children ; count ++)
 	{
 		if( win->children[count]->is_mapped)
@@ -331,7 +356,7 @@ total_base_height(
 	u_int  base_height ;
 
 	base_height = win->base_height + win->margin * 2 ;
-	
+
 	for( count = 0 ; count < win->num_of_children ; count ++)
 	{
 		if( win->children[count]->is_mapped)
@@ -352,7 +377,7 @@ total_width_inc(
 	u_int  width_inc ;
 
 	width_inc = win->width_inc ;
-	
+
 	for( count = 0 ; count < win->num_of_children ; count ++)
 	{
 		if( win->children[count]->is_mapped)
@@ -382,7 +407,7 @@ total_height_inc(
 	u_int  height_inc ;
 
 	height_inc = win->height_inc ;
-	
+
 	for( count = 0 ; count < win->num_of_children ; count ++)
 	{
 		if( win->children[count]->is_mapped)
@@ -422,17 +447,17 @@ x_window_init(
 	/*
 	 * these values are set later
 	 */
-	 
+
 	win->display = NULL ;
 	win->screen = 0 ;
-	
+
 	win->parent_window = None ;
 	win->my_window = None ;
-	
+
 	win->drawable = None ;
 	win->pixmap = None ;
-	
-#ifdef  ANTI_ALIAS
+
+#ifdef  USE_TYPE_XFT
 	win->xft_draw = NULL ;
 #endif
 
@@ -441,7 +466,7 @@ x_window_init(
 
 	win->fg_color = 0 ;
 	win->bg_color = 0 ;
-	
+
 	win->parent = NULL ;
 	win->children = NULL ;
 	win->num_of_children = 0 ;
@@ -449,10 +474,10 @@ x_window_init(
 	win->use_pixmap = 0 ;
 
 	win->pic_mod = NULL ;
-	
+
 	win->wall_picture_is_set = 0 ;
 	win->is_transparent = 0 ;
-	
+
 	win->cursor_shape = 0 ;
 
 	win->event_mask = ExposureMask | VisibilityChangeMask | FocusChangeMask | PropertyChangeMask ;
@@ -482,9 +507,9 @@ x_window_init(
 	win->prev_clicked_button = -1 ;
 	win->click_num = 0 ;
 	win->button_is_pressing = 0 ;
-
+#ifndef  DISABLE_XDND
 	win->dnd = NULL ;
-
+#endif
 	win->app_name = "mlterm" ;
 
 	win->window_realized = NULL ;
@@ -504,9 +529,12 @@ x_window_init(
 	win->utf8_selection_requested = NULL ;
 	win->xct_selection_notified = NULL ;
 	win->utf8_selection_notified = NULL ;
-	win->set_xdnd_config = NULL ;
 	win->window_deleted = NULL ;
-	
+	win->mapping_notify = NULL ;
+#ifndef  DISABLE_XDND
+	win->set_xdnd_config = NULL ;
+#endif
+
 	return	1 ;
 }
 
@@ -521,7 +549,7 @@ x_window_final(
 	kik_debug_printf( "[deleting child windows]\n") ;
 	x_window_dump_children( win) ;
 #endif
-	
+
 	for( count = 0 ; count < win->num_of_children ; count ++)
 	{
 		x_window_final( win->children[count]) ;
@@ -531,9 +559,9 @@ x_window_final(
 	{
 		free( win->children) ;
 	}
-	
+
 	x_window_manager_clear_selection( x_get_root_window( win)->win_man , win) ;
-	
+
 	XFreeGC( win->display , win->gc) ;
 	XFreeGC( win->display , win->ch_gc) ;
 
@@ -542,14 +570,14 @@ x_window_final(
 		XFreePixmap( win->display , win->pixmap) ;
 	}
 
-#ifdef  ANTI_ALIAS
+#ifdef  USE_TYPE_XFT
 	XftDrawDestroy( win->xft_draw) ;
 #endif
-	
+
 	XDestroyWindow( win->display , win->my_window) ;
 
 	x_xic_deactivate( win) ;
-	
+
 	if( win->window_finalized)
 	{
 		(*win->window_finalized)( win) ;
@@ -567,7 +595,7 @@ x_window_init_event_mask(
 	if( win->my_window)
 	{
 		/* this can be used before x_window_show() */
-		
+
 		return  0 ;
 	}
 
@@ -640,7 +668,7 @@ x_window_set_wall_picture(
 		/*
 		 * wall picture can not be used in double buffering mode.
 		 */
-		
+
 		return  0 ;
 	}
 
@@ -649,7 +677,7 @@ x_window_set_wall_picture(
 		/*
 		 * unset transparent before setting wall picture !
 		 */
-		 
+
 		return  0 ;
 	}
 
@@ -662,7 +690,7 @@ x_window_set_wall_picture(
 	}
 
 	XSetWindowBackgroundPixmap( win->display , win->my_window , pic) ;
-	
+
 	win->wall_picture_is_set = 1 ;
 
 	if( win->window_exposed)
@@ -684,14 +712,14 @@ x_window_unset_wall_picture(
 		/*
 		 * wall picture can not be used in double buffering mode.
 		 */
-		 
+
 		return  0 ;
 	}
 
 	if( ! win->wall_picture_is_set)
 	{
 		/* already unset */
-		
+
 		return  1 ;
 	}
 
@@ -701,7 +729,7 @@ x_window_unset_wall_picture(
 		 * transparent background is not a wall picture :)
 		 * this case is regarded as not using a wall picture.
 		 */
-	
+
 		return  1 ;
 	}
 
@@ -714,10 +742,10 @@ x_window_unset_wall_picture(
 		/* setting 0 in case the current status is VisibilityPartiallyObscured. */
 		win->is_scrollable = 0 ;
 	}
-	
+
 	XSetWindowBackgroundPixmap( win->display , win->my_window , None) ;
 	XSetWindowBackground( win->display , win->my_window , win->bg_color) ;
-	
+
 	win->wall_picture_is_set = 0 ;
 
 	if( win->window_exposed)
@@ -742,7 +770,7 @@ x_window_set_transparent(
 	int  count ;
 
 	win->pic_mod = pic_mod ;
-	
+
 	if( win->my_window == None)
 	{
 		/*
@@ -750,7 +778,7 @@ x_window_set_transparent(
 		 * ReparentNotify event will do transparent processing automatically after
 		 * x_window_show().
 		 */
-		 
+
 		win->is_transparent = 1 ;
 
 		goto  end ;
@@ -793,7 +821,7 @@ end:
 	{
 		x_window_set_transparent( win->children[count] , win->pic_mod) ;
 	}
-	
+
 	return  1 ;
 }
 
@@ -809,7 +837,7 @@ x_window_unset_transparent(
 	Window *  list ;
 	int  n ;
 #endif
-	
+
 	if( ! win->is_transparent)
 	{
 		/* already unset */
@@ -838,18 +866,18 @@ x_window_unset_transparent(
 		XSetWindowBackgroundPixmap( win->display , parent , None) ;
 	}
 #endif
-	
+
 	if( win->window_exposed)
 	{
 		x_window_clear_all( win) ;
 		(*win->window_exposed)( win , 0 , 0 , win->width , win->height) ;
 	}
-	
+
 	for( count = 0 ; count < win->num_of_children ; count ++)
 	{
 		x_window_unset_transparent( win->children[count]) ;
 	}
-	
+
 	return  1 ;
 }
 
@@ -862,7 +890,7 @@ x_window_use_pixmap(
 	)
 {
 	win->use_pixmap = 1 ;
-	
+
 	win->event_mask &= ~VisibilityChangeMask ;
 
 	/* always true */
@@ -922,14 +950,14 @@ x_window_add_child(
 	)
 {
 	void *  p ;
-	
+
 	if( ( p = realloc( win->children , sizeof( *win->children) * (win->num_of_children + 1)))
 		== NULL)
 	{
 	#ifdef  DEBUG
 		kik_warn_printf( KIK_DEBUG_TAG " realloc failed.\n") ;
 	#endif
-	
+
 		return  0 ;
 	}
 
@@ -943,7 +971,7 @@ x_window_add_child(
 	child->x = x ;
 	child->y = y ;
 	child->is_mapped = map ;
-	
+
 	win->children[ win->num_of_children ++] = child ;
 
 	return  1 ;
@@ -974,7 +1002,7 @@ x_window_show(
 	if( win->my_window)
 	{
 		/* already shown */
-		
+
 		return  0 ;
 	}
 
@@ -1018,7 +1046,7 @@ x_window_show(
 			XCreateFontCursor( win->display , win->cursor_shape)) ;
 	}
 
-#ifdef  ANTI_ALIAS
+#ifdef  USE_TYPE_XFT
 	win->xft_draw = XftDrawCreate( win->display , win->drawable ,
 					DefaultVisual( win->display , win->screen) ,
 					DefaultColormap( win->display , win->screen)) ;
@@ -1042,8 +1070,8 @@ x_window_show(
 		XWMHints  wm_hints ;
 		int  argc = 1 ;
 		char *  argv[] = { "mlterm" , NULL , } ;
-		Atom  protocols[1] ;
-		
+		Atom  protocols[2] ;
+
 		win->event_mask |= StructureNotifyMask ;
 
 		/*
@@ -1054,7 +1082,7 @@ x_window_show(
 		size_hints.y = win->y ;
 		size_hints.width = ACTUAL_WIDTH(win) ;
 		size_hints.height = ACTUAL_HEIGHT(win) ;
-		
+
 		size_hints.width_inc = total_width_inc( win) ;
 		size_hints.height_inc = total_height_inc( win) ;
 		size_hints.min_width = total_min_width( win) ;
@@ -1086,7 +1114,7 @@ x_window_show(
 		}
 
 		size_hints.flags = PSize | PMinSize | PResizeInc | PBaseSize | PWinGravity ;
-		
+
 		if( hint & (XValue | YValue))
 		{
 			size_hints.flags |= PPosition ;
@@ -1115,8 +1143,9 @@ x_window_show(
 	#endif
 
 		protocols[0] = XA_DELETE_WINDOW(win->display) ;
-		
-		XSetWMProtocols( win->display , win->my_window , protocols , 1) ;
+		protocols[1] = XA_TAKE_FOCUS(win->display) ;
+
+		XSetWMProtocols( win->display , win->my_window , protocols , 2) ;
 	}
 
 	/*
@@ -1133,7 +1162,7 @@ x_window_show(
 #if  0
 	{
 		char *  locale ;
-		
+
 		if( ( locale = kik_get_locale()))
 		{
 			XChangeProperty( win->display , win->my_window ,
@@ -1156,11 +1185,11 @@ x_window_show(
 	/*
 	 * really visualized.
 	 */
-	 
+
 	if( win->is_mapped)
 	{
 		XMapWindow( win->display , win->my_window) ;
-		
+
 	#if  0
 		x_window_clear_all( win) ;
 	#endif
@@ -1194,7 +1223,7 @@ x_window_unmap(
 	{
 		return  1 ;
 	}
-	
+
 	XUnmapWindow( win->display , win->my_window) ;
 	win->is_mapped = 0 ;
 
@@ -1213,10 +1242,10 @@ x_window_resize(
 	{
 		return  0 ;
 	}
-	
+
 	win->width = width ;
 	win->height = height ;
-	
+
 	if( win->pixmap)
 	{
 		XFreePixmap( win->display , win->pixmap) ;
@@ -1225,7 +1254,7 @@ x_window_resize(
 			DefaultDepth( win->display , win->screen)) ;
 		win->drawable = win->pixmap ;
 
-	#ifdef  ANTI_ALIAS
+	#ifdef  USE_TYPE_XFT
 		XftDrawChange( win->xft_draw , win->drawable) ;
 	#endif
 	}
@@ -1234,9 +1263,9 @@ x_window_resize(
 	{
 		(*win->parent->child_window_resized)( win->parent , win) ;
 	}
-	
+
 	XResizeWindow( win->display , win->my_window , ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win)) ;
-	
+
 	if( (flag & NOTIFY_TO_MYSELF) && win->window_resized)
 	{
 		(*win->window_resized)( win) ;
@@ -1296,7 +1325,92 @@ x_window_set_normal_hints(
 	size_hints.flags = PMinSize | PResizeInc | PBaseSize ;
 
 	XSetWMNormalHints( root->display , root->my_window , &size_hints) ;
-		
+
+	return  1 ;
+}
+
+int
+x_window_set_override_redirect(
+	x_window_t *  win ,
+	int  flag
+	)
+{
+	x_window_t *  root ;
+	XSetWindowAttributes  s_attr ;
+	XWindowAttributes  g_attr ;
+
+	root = x_get_root_window(win) ;
+
+	XGetWindowAttributes( root->display , root->my_window , &g_attr) ;
+	if( flag)
+	{
+		s_attr.override_redirect = True ;
+	}
+	else
+	{
+		s_attr.override_redirect = False ;
+	}
+
+	if( g_attr.override_redirect == s_attr.override_redirect)
+	{
+		return  1 ;
+	}
+
+	XChangeWindowAttributes( root->display , root->my_window ,
+				 CWOverrideRedirect , &s_attr) ;
+
+	if( g_attr.map_state != IsUnmapped)
+	{
+		XUnmapWindow( root->display , root->my_window) ;
+		XMapWindow( root->display , root->my_window) ;
+	}
+
+	return  1 ;
+}
+
+int
+x_window_set_borderless_flag(
+	x_window_t *  win ,
+	int  flag
+	)
+{
+	/*
+	 * XXX
+	 * Support borderless with _MOTIF_WM_HINTS.
+	 * (See Eterm/src/windows.c)
+	 */
+
+	x_window_t *  root ;
+	Atom  atom ;
+
+	root = x_get_root_window(win) ;
+
+#ifdef  __DEBUG
+	kik_debug_printf( "MOTIF_WM_HINTS: %x\nMOTIF_WM_INFO: %x\n" ,
+		XInternAtom( root->display , "_MOTIF_WM_HINTS" , True) ,
+		XInternAtom( root->display , "_MOTIF_WM_INFO" , True)) ;
+#endif
+
+	if( ( atom = XA_MWM_HINTS(root->display)) != None)
+	{
+		if( flag)
+		{
+			MWMHints_t  mwmhints = { MWM_HINTS_DECORATIONS, 0, 0, 0, 0 } ;
+
+			XChangeProperty( root->display, root->my_window, atom , atom , 32,
+				PropModeReplace, (u_char *)&mwmhints, sizeof(MWMHints_t)/4) ;
+		}
+		else
+		{
+			XDeleteProperty( root->display, root->my_window, atom) ;
+		}
+	}
+	else
+	{
+		/* fall back to override redirect */
+		x_window_set_override_redirect( win , flag) ;
+	}
+
 	return  1 ;
 }
 
@@ -1345,7 +1459,7 @@ x_window_clear_margin_area(
 	if( win->drawable == win->pixmap)
 	{
 		XSetForeground( win->display , win->gc , win->bg_color) ;
-		
+
 		XFillRectangle( win->display , win->drawable , win->gc ,
 			0 , 0 , win->margin , ACTUAL_HEIGHT(win)) ;
 		XFillRectangle( win->display , win->drawable , win->gc ,
@@ -1354,7 +1468,7 @@ x_window_clear_margin_area(
 			win->width + win->margin , 0 , win->margin , ACTUAL_HEIGHT(win)) ;
 		XFillRectangle( win->display , win->drawable , win->gc ,
 			win->margin , win->height + win->margin , win->width , win->margin) ;
-		
+
 		XSetForeground( win->display , win->gc , win->fg_color) ;
 	}
 	else if( win->drawable == win->my_window)
@@ -1422,7 +1536,7 @@ x_window_fill_with(
 	{
 		XSetForeground( win->display , win->gc , color) ;
 	}
-	
+
 	XFillRectangle( win->display , win->drawable , win->gc , x + win->margin , y + win->margin ,
 		width , height) ;
 
@@ -1455,7 +1569,7 @@ x_window_fill_all_with(
 	{
 		XSetForeground( win->display , win->gc , color) ;
 	}
-	
+
 	XFillRectangle( win->display , win->drawable , win->gc ,
 		0 , 0 , ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win)) ;
 
@@ -1503,7 +1617,7 @@ x_window_update_view(
 				" update size (y)%d (h)%d is overflowed.(screen height is %d)\n" ,
 				y , height , win->height) ;
 		#endif
-				
+
 			height = win->height - y ;
 
 		#ifdef  DEBUG
@@ -1537,7 +1651,7 @@ x_window_idling(
 	)
 {
 	int  count ;
-	
+
 	for( count = 0 ; count < win->num_of_children ; count ++)
 	{
 		x_window_idling( win->children[count]) ;
@@ -1563,7 +1677,7 @@ x_window_receive_event(
 	)
 {
 	int  count ;
-	
+
 	for( count = 0 ; count < win->num_of_children ; count ++)
 	{
 		if( x_window_receive_event( win->children[count] , event))
@@ -1571,7 +1685,6 @@ x_window_receive_event(
 			return  1 ;
 		}
 	}
-
 	if( win->my_window != event->xany.window)
 	{
 		/*
@@ -1580,7 +1693,7 @@ x_window_receive_event(
 		 * no xim window will be opened at XFilterEvent() in x_window_manager_receive_next_event().
 		 * but it is desired to open xim window of x_screen when its event is invoked
 		 * on scrollbar or title bar.
-		 * this hack enables it , but this way won't deal with the case that multiple 
+		 * this hack enables it , but this way won't deal with the case that multiple
 		 * xics exist.
 		 */
 		if( win->xic)
@@ -1602,15 +1715,33 @@ x_window_receive_event(
 			return  1 ;
 		}
 
+		if( event->type == MappingNotify && event->xmapping.request != MappingPointer)
+		{
+			if( win->win_man)
+			{
+#ifdef  DEBUG
+				kik_warn_printf( KIK_DEBUG_TAG
+						 " MappingNotify serial #%d\n", event->xmapping.serial) ;
+#endif
+				XRefreshKeyboardMapping( &(event->xmapping));
+				x_window_manager_update_modifier_mapping( win->win_man, event->xmapping.serial) ;
+				/* have to  rocess only once */
+				return 1 ;
+			}
+
+			if( win->mapping_notify){
+				(*win->mapping_notify)( win);
+			}
+		}
 		return  0 ;
 	}
-
+#ifndef  DISABLE_XDND
 	if( x_dnd_filter_event( event, win) != 0)
 	{
 		/* event was consumed by xdnd handlers */
 		return 1 ;
 	}
-
+#endif
 	if( event->type == KeyPress)
 	{
 		if( win->key_pressed)
@@ -1649,7 +1780,7 @@ x_window_receive_event(
 	else if( event->type == MotionNotify)
 	{
 		XEvent  ahead ;
-		
+
 		while( XEventsQueued(win->display , QueuedAfterReading))
 		{
 			XPeekEvent( win->display , &ahead) ;
@@ -1664,7 +1795,7 @@ x_window_receive_event(
 
 		event->xmotion.x -= win->margin ;
 		event->xmotion.y -= win->margin ;
-	
+
 		if( win->button_motion)
 		{
 			(*win->button_motion)( win , &event->xmotion) ;
@@ -1683,7 +1814,7 @@ x_window_receive_event(
 	{
 		event->xbutton.x -= win->margin ;
 		event->xbutton.y -= win->margin ;
-		
+
 		if( win->button_released)
 		{
 			(*win->button_released)( win , &event->xbutton) ;
@@ -1713,7 +1844,7 @@ x_window_receive_event(
 			win->prev_clicked_time = event->xbutton.time ;
 			win->prev_clicked_button = event->xbutton.button ;
 		}
-		
+
 		if( win->button_pressed)
 		{
 			(*win->button_pressed)( win , &event->xbutton , win->click_num) ;
@@ -1808,7 +1939,7 @@ x_window_receive_event(
 		}
 	}
 	else if( event->type == ConfigureNotify)
-	{		
+	{
 		int  is_changed ;
 		XEvent  next_ev ;
 
@@ -1822,8 +1953,8 @@ x_window_receive_event(
 			if( abs( event->xconfigure.x - win->x) % DisplayWidth(win->display , win->screen) != 0 ||
 				abs( event->xconfigure.y - win->y) % DisplayHeight(win->display , win->screen) != 0 ||
 				( event->xconfigure.x < 0 && event->xconfigure.x + (int)ACTUAL_WIDTH(win) > 0) ||
-				( event->xconfigure.x > 0 && 
-					event->xconfigure.x + (int)ACTUAL_WIDTH(win) 
+				( event->xconfigure.x > 0 &&
+					event->xconfigure.x + (int)ACTUAL_WIDTH(win)
 						> DisplayWidth( win->display , win->screen)) ||
 				( event->xconfigure.y < 0 && event->xconfigure.y + (int)ACTUAL_HEIGHT(win) > 0) ||
 				( event->xconfigure.y > 0 &&
@@ -1851,7 +1982,7 @@ x_window_receive_event(
 					DefaultDepth( win->display , win->screen)) ;
 				win->drawable = win->pixmap ;
 
-			#ifdef  ANTI_ALIAS
+			#ifdef  USE_TYPE_XFT
 				XftDrawChange( win->xft_draw , win->drawable) ;
 			#endif
 			}
@@ -1863,15 +1994,13 @@ x_window_receive_event(
 				(*win->window_resized)( win) ;
 			}
 
-			x_xic_resized( win) ;
-
 			is_changed = 1 ;
 		}
 
 		/*
 		 * transparent processing.
 		 */
-		 
+
 		if( is_changed &&
 			XCheckMaskEvent( win->display , StructureNotifyMask , &next_ev))
 		{
@@ -1882,7 +2011,7 @@ x_window_receive_event(
 
 			XPutBackEvent( win->display , &next_ev) ;
 		}
-		
+
 		if( is_changed)
 		{
 			notify_configure_to_children( win) ;
@@ -1912,18 +2041,26 @@ x_window_receive_event(
 	{
 		Atom  xa_utf8_string ;
 		Atom  xa_compound_text ;
+	#ifdef  DEBUG
+		Atom  xa_multiple ;
+	#endif
+		Atom  xa_targets ;
 		Atom  xa_text ;
-		
+
 		xa_compound_text = XA_COMPOUND_TEXT(win->display) ;
+		xa_targets = XA_TARGETS(win->display) ;
+#ifdef  DEBUG
+		xa_multiple = XA_MULTIPLE(win->display) ;
+#endif
 		xa_text = XA_TEXT(win->display) ;
 		xa_utf8_string = XA_UTF8_STRING(win->display) ;
-		
+
 		if( event->xselectionrequest.target == XA_STRING)
 		{
 			if( win->xct_selection_requested)
 			{
 				(*win->xct_selection_requested)( win , &event->xselectionrequest ,
-					event->xselectionrequest.target) ; 
+					event->xselectionrequest.target) ;
 			}
 		}
 		else if( event->xselectionrequest.target == xa_text ||
@@ -1936,9 +2073,9 @@ x_window_receive_event(
 				 * wants it to be sent back with "COMPOUND_TEXT" atom.
 				 * why ?
 				 */
-				 
+
 				(*win->xct_selection_requested)( win , &event->xselectionrequest ,
-					xa_compound_text) ; 
+					xa_compound_text) ;
 			}
 		}
 		else if( event->xselectionrequest.target == xa_utf8_string)
@@ -1949,6 +2086,22 @@ x_window_receive_event(
 					xa_utf8_string) ;
 			}
 		}
+		else if( event->xselectionrequest.target == xa_targets)
+		{
+			Atom  targets[4] ;
+
+			targets[0] =XA_STRING ;
+			targets[1] =xa_text ;
+			targets[2] =xa_compound_text ;
+			targets[3] =xa_utf8_string ;
+			x_window_send_selection( win , &event->xselectionrequest , (u_char *)(&targets) , sizeof(targets) , XA_ATOM) ;
+		}
+#ifdef  DEBUG
+		else if( event->xselectionrequest.target == xa_multiple)
+		{
+			kik_debug_printf("MULTIPLE requested(not yet implemented)\n") ;
+		}
+#endif
 		else
 		{
 			x_window_send_selection( win , &event->xselectionrequest , NULL , 0 , 0) ;
@@ -1960,12 +2113,12 @@ x_window_receive_event(
 		Atom  xa_compound_text ;
 		Atom  xa_text ;
 		Atom  xa_selection ;
-		
+
 		xa_compound_text = XA_COMPOUND_TEXT(win->display) ;
 		xa_text = XA_TEXT(win->display) ;
 		xa_utf8_string = XA_UTF8_STRING(win->display) ;
 		xa_selection = XA_SELECTION(win->display) ;
-		
+
 		if( event->xselection.property == None)
 		{
 			/*
@@ -1988,7 +2141,7 @@ x_window_receive_event(
 				XConvertSelection( win->display , XA_PRIMARY , XA_STRING ,
 					xa_selection , win->my_window , CurrentTime) ;
 			}
-			
+
 			return  1 ;
 		}
 
@@ -2018,15 +2171,15 @@ x_window_receive_event(
 				{
 					break ;
 				}
-				
+
 				if( ct.value == NULL || ct.nitems == 0)
 				{
 					break ;
 				}
 
-				if( event->xselection.target == XA_STRING || 
-					 event->xselection.target == xa_text ||
-					 event->xselection.target == xa_compound_text)
+				if( ct.encoding == XA_STRING ||
+					 ct.encoding == xa_text ||
+					 ct.encoding == xa_compound_text)
 				{
 					if( win->xct_selection_notified)
 					{
@@ -2034,7 +2187,7 @@ x_window_receive_event(
 							win , ct.value , ct.nitems) ;
 					}
 				}
-				else if( event->xselection.target == xa_utf8_string)
+				else if( ct.encoding == xa_utf8_string)
 				{
 					if( win->utf8_selection_notified)
 					{
@@ -2044,22 +2197,22 @@ x_window_receive_event(
 				}
 
 				XFree( ct.value) ;
-				
+
 				if( bytes_after == 0)
 				{
 					break ;
 				}
 			}
-			
+
 		}
-		
+
 		XDeleteProperty( win->display, event->xselection.requestor,
 			event->xselection.property) ;
 	}
 	else if( event->type == VisibilityNotify)
 	{
 		/* this event is changeable in run time. */
-		
+
 		if( win->event_mask & VisibilityChangeMask)
 		{
 			if( event->xvisibility.state == VisibilityPartiallyObscured)
@@ -2089,7 +2242,71 @@ x_window_receive_event(
 			{
 				exit(0) ;
 			}
-		}		
+		}
+#if 0
+		if( event->xclient.format == 32 &&
+			event->xclient.data.l[0] == XA_TAKE_FOCUS( win->display))
+		{
+			kik_warn_printf( KIK_DEBUG_TAG
+				" TakeFocus message is received.\n") ;
+		}
+#endif
+	}
+	else if( event->type == PropertyNotify)
+	{
+
+		if( event->xproperty.atom == XA_SELECTION( win->display) &&
+		    event->xproperty.state == PropertyNewValue)
+		{
+			XTextProperty  ct ;
+			u_long  bytes_after ;
+
+			XGetWindowProperty( win->display, event->xproperty.window,
+					    event->xproperty.atom, 0, 0, False,
+					    AnyPropertyType, &ct.encoding, &ct.format,
+					    &ct.nitems, &bytes_after, &ct.value) ;
+			if( ct.value)
+			{
+				XFree( ct.value) ;
+			}
+
+			if( ct.encoding == XA_INCR(win->display)
+			    || bytes_after == 0)
+			{
+				XDeleteProperty( win->display, event->xproperty.window,
+						 ct.encoding) ;
+			}
+			else
+			{
+				XGetWindowProperty( win->display , event->xproperty.window ,
+						    event->xproperty.atom , 0 , bytes_after , True ,
+						    AnyPropertyType , &ct.encoding , &ct.format ,
+						    &ct.nitems , &bytes_after , &ct.value) ;
+				if(ct.encoding == XA_STRING ||
+				   ct.encoding == XA_TEXT(win->display) ||
+				   ct.encoding == XA_COMPOUND_TEXT(win->display))
+				{
+					if( win->xct_selection_notified)
+					{
+						(*win->xct_selection_notified)(
+							win , ct.value , ct.nitems) ;
+					}
+				}
+				else if(ct.encoding == XA_UTF8_STRING(win->display))
+				{
+					if( win->utf8_selection_notified)
+					{
+						(*win->utf8_selection_notified)(
+							win , ct.value , ct.nitems) ;
+					}
+				}
+
+				if( ct.value)
+				{
+					XFree( ct.value) ;
+				}
+			}
+		}
 	}
 #ifdef  __DEBUG
 	else
@@ -2120,7 +2337,7 @@ x_window_get_str(
 	{
 		return  len ;
 	}
-	 
+
 	if( ( len = XLookupString( event , seq , seq_len , keysym , NULL)) > 0)
 	{
 		*parser = NULL ;
@@ -2148,7 +2365,7 @@ x_window_is_scrollable(
  * Scroll functions.
  * The caller side should clear the scrolled area.
  */
- 
+
 int
 x_window_scroll_upward(
 	x_window_t *  win ,
@@ -2170,7 +2387,7 @@ x_window_scroll_upward_region(
 	{
 		return  0 ;
 	}
-	
+
 	if( boundary_start < 0 || boundary_end > win->height || boundary_end <= boundary_start + height)
 	{
 	#ifdef  DEBUG
@@ -2178,10 +2395,10 @@ x_window_scroll_upward_region(
 			" boundary start %d end %d height %d in window((h) %d (w) %d)\n" ,
 			boundary_start , boundary_end , height , win->height , win->width) ;
 	#endif
-	
+
 		return  0 ;
 	}
-	
+
 	XCopyArea( win->display , win->drawable , win->drawable , win->gc ,
 		win->margin , win->margin + boundary_start + height ,	/* src */
 		win->width , boundary_end - boundary_start - height ,	/* size */
@@ -2218,7 +2435,7 @@ x_window_scroll_downward_region(
 		kik_warn_printf( KIK_DEBUG_TAG " boundary start %d end %d height %d\n" ,
 			boundary_start , boundary_end , height) ;
 	#endif
-	
+
 		return  0 ;
 	}
 
@@ -2251,7 +2468,7 @@ x_window_scroll_leftward_region(
 	{
 		return  0 ;
 	}
-	
+
 	if( boundary_start < 0 || boundary_end > win->width || boundary_end <= boundary_start + width)
 	{
 	#ifdef  DEBUG
@@ -2259,10 +2476,10 @@ x_window_scroll_leftward_region(
 			" boundary start %d end %d width %d in window((h) %d (w) %d)\n" ,
 			boundary_start , boundary_end , width , win->height , win->width) ;
 	#endif
-	
+
 		return  0 ;
 	}
-	
+
 	XCopyArea( win->display , win->drawable , win->drawable , win->gc ,
 		win->margin + boundary_start + width , win->margin ,	/* src */
 		boundary_end - boundary_start - width , win->height ,	/* size */
@@ -2299,7 +2516,7 @@ x_window_scroll_rightward_region(
 		kik_warn_printf( KIK_DEBUG_TAG " boundary start %d end %d width %d\n" ,
 			boundary_start , boundary_end , width) ;
 	#endif
-	
+
 		return  0 ;
 	}
 
@@ -2309,7 +2526,7 @@ x_window_scroll_rightward_region(
 		win->margin + boundary_start + width , win->margin) ;
 
 	return  1 ;
-}	
+}
 
 int
 x_window_draw_decsp_string(
@@ -2345,6 +2562,7 @@ x_window_draw_decsp_string(
 		}
 		return  1 ;
 	}
+#ifdef  USE_TYPE_XCORE
 	else if( font->xfont)
 	{
 		if( bg_color)
@@ -2357,12 +2575,14 @@ x_window_draw_decsp_string(
 			return  x_window_draw_string( win , font , fg_color , x , y , str , len) ;
 		}
 	}
+#endif
 	else
 	{
 		return  0 ;
 	}
 }
 
+#ifdef  USE_TYPE_XCORE
 int
 x_window_draw_string(
 	x_window_t *  win ,
@@ -2458,7 +2678,7 @@ x_window_draw_image_string16(
 	XSetFont( win->display , win->ch_gc , font->xfont->fid) ;
 	XSetForeground( win->display , win->ch_gc , fg_color->pixel) ;
 	XSetBackground( win->display , win->ch_gc , bg_color->pixel) ;
-	
+
 	XDrawImageString16( win->display , win->drawable , win->ch_gc ,
 		x + font->x_off + win->margin , y + win->margin , str , len) ;
 
@@ -2470,8 +2690,9 @@ x_window_draw_image_string16(
 
 	return  1 ;
 }
+#endif
 
-#ifdef  ANTI_ALIAS
+#ifdef  USE_TYPE_XFT
 int
 x_window_xft_draw_string8(
 	x_window_t *  win ,
@@ -2514,7 +2735,7 @@ x_window_xft_draw_string32(
 		XftDrawString32( win->xft_draw , fg_color , font->xft_font ,
 			x + win->margin + 1 , y + win->margin , str , len) ;
 	}
-	
+
 	return  1 ;
 }
 #endif
@@ -2543,7 +2764,7 @@ x_window_set_selection_owner(
 	)
 {
 	XSetSelectionOwner( win->display , XA_PRIMARY , win->my_window , time) ;
-	
+
 	if( win->my_window != XGetSelectionOwner( win->display , XA_PRIMARY))
 	{
 		return  0 ;
@@ -2607,7 +2828,7 @@ x_window_send_selection(
 			8 , PropModeReplace , sel_str , sel_len) ;
 		res_ev.xselection.property = req_ev->property ;
 	}
-	
+
 	XSendEvent( win->display , res_ev.xselection.requestor , False , 0 , &res_ev) ;
 
 	return  1 ;
@@ -2623,7 +2844,7 @@ x_set_window_name(
 	XTextProperty  prop ;
 
 	root = x_get_root_window( win) ;
-	
+
 	if( name == NULL)
 	{
 		name = root->app_name ;
@@ -2679,10 +2900,10 @@ x_set_icon_name(
 
 int
 x_window_set_icon(
- 	x_window_t *  win,
- 	Pixmap  icon,
- 	Pixmap  mask,
- 	u_int32_t *  cardinal	
+	x_window_t *  win,
+	Pixmap  icon,
+	Pixmap  mask,
+	u_int32_t *  cardinal
 	)
 {
 	XWMHints *hints ;
@@ -2695,11 +2916,11 @@ x_window_set_icon(
 				 XA_NET_WM_ICON( win->display),
 				 XA_CARDINAL, 32, PropModeReplace,
 				 (unsigned char *)(cardinal),
-				 /* (cardinal[0])*(cardinal[1]) 
+				 /* (cardinal[0])*(cardinal[1])
 				  *          = width * height */
-				 (cardinal[0])*(cardinal[1]) +2) ; 
+				 (cardinal[0])*(cardinal[1]) +2) ;
 	}
-	/* set old style window manager hint's icon */		
+	/* set old style window manager hint's icon */
 	hints = NULL ;
 	if (icon || mask)
 	{
@@ -2722,7 +2943,7 @@ x_window_set_icon(
 		hints->flags |= IconMaskHint ;
 		hints->icon_mask = mask ;
 	}
-	
+
 	XSetWMHints( win->display, win->my_window, hints) ;
 	XFree( hints) ;
 
@@ -2743,7 +2964,7 @@ x_window_get_visible_geometry(
 	Window  child ;
 	int screen_width ;
 	int screen_height ;
-	
+
 	XTranslateCoordinates( win->display , win->my_window ,
 		DefaultRootWindow( win->display) , 0 , 0 ,
 		x , y , &child) ;
@@ -2754,7 +2975,7 @@ x_window_get_visible_geometry(
 	if( *x >= screen_width || *y >= screen_height)
 	{
 		/* no visible window */
-		
+
 		return  0 ;
 	}
 
@@ -2763,12 +2984,12 @@ x_window_get_visible_geometry(
 		if( ACTUAL_WIDTH(win) <= abs(*x))
 		{
 			/* no visible window */
-			
+
 			return  0 ;
 		}
 
 		*my_x = abs(*x) ;
-		
+
 		*width = ACTUAL_WIDTH(win) - abs(*x) ;
 		*x = 0 ;
 	}
@@ -2783,7 +3004,7 @@ x_window_get_visible_geometry(
 		if( ACTUAL_HEIGHT(win) <= abs(*y))
 		{
 			/* no visible window */
-			
+
 			return  0 ;
 		}
 
@@ -2821,6 +3042,14 @@ x_set_click_interval(
 	return  1 ;
 }
 
+XModifierKeymap *
+x_window_get_modifier_mapping(
+	x_window_t *  win
+	)
+{
+	return  x_window_manager_get_modifier_mapping( win->win_man) ;
+}
+
 #if  0
 /*
  * XXX
@@ -2882,7 +3111,7 @@ x_window_dump_children(
 	for( count = 0 ; count < win->num_of_children ; count ++)
 	{
 		kik_msg_printf( "%p(%li) " , win->children[count] ,
-			win->children[count]->my_window) ; 
+			win->children[count]->my_window) ;
 	}
 	kik_msg_printf( "\n") ;
 
